/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1997 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the MASH Research
 * 	Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char rcsid[] =
    "@(#) $Header: /cvsroot/nsnam/ns-2/apps/rtp.cc,v 1.27 2005/08/22 05:08:32 tomh Exp $";
#endif


#include <stdlib.h>

#include "config.h"
#include "agent.h"
#include "random.h"
#include "rtp.h"

#define SAMPLERATE 8000

int hdr_rtp::offset_;

class RTPHeaderClass : public PacketHeaderClass {
public: 
        RTPHeaderClass() : PacketHeaderClass("PacketHeader/RTP",
					     sizeof(hdr_rtp)) {
		bind_offset(&hdr_rtp::offset_);
	}
} class_rtphdr;

static class RTPAgentClass : public TclClass {
public:
	RTPAgentClass() : TclClass("Agent/RTP") {}
	TclObject* create(int, const char*const*) {
		return (new RTPAgent());
	}
} class_rtp_agent;

RTPAgent::RTPAgent() : Agent(PT_RTP), session_(0), lastpkttime_(-1e6), id_(0), openfile(0)
{
	bind("seqno_", &seqno_);
	bind("packetSize_", &size_);
	//bind("frame_type_",&frame_type_);
	
}



void RTPAgent::sendmsg(int nbytes, int frame_number)
{
//	printf(" RTPAgent::sendmsg() session_ %d \n", session_->srcid());
	
        Packet *p;
	int n;
	char buf[100]; //added by smallko

	if (size_)
		n = nbytes / size_;
	else
		printf("Error: myUDP size = 0\n");

	if (nbytes == -1) {
		printf("Error:  sendmsg() for UDP should not be -1\n");
		return;
	}	

	
//	double local_time = Scheduler::instance().clock();
	while (n-- > 0) {
		p = allocpkt();
		double local_time = Scheduler::instance().clock();
		hdr_cmn::access(p)->size() = size_;
		hdr_cmn::access(p)->frametype_=frametype_;
		hdr_cmn::access(p)->frame_number = frame_number;
		hdr_cmn::access(p)->quant = quant_;
		hdr_rtp* rh = hdr_rtp::access(p);
		rh->srcid() = session_->srcid();
		set_pkttype(PT_VIDEO);
		rh->flags() = 0;
		rh->seqno() = ++seqno_;
		
		
		rh->srcid() = session_ ? session_->srcid() : 0;
		if (session_) {
 			session_->localsrc_update(size_);
 			
		}
		
		
		hdr_cmn::access(p)->timestamp() = 
		    (u_int32_t)(SAMPLERATE*local_time);
		hdr_cmn::access(p)->sendtime_ = local_time;	// (smallko)
		if(openfile!=0){
			hdr_cmn::access(p)->frame_pkt_id_ = id_++;

 		sprintf(buf, "%-16f %d %d %-16d %-8d %-8d \n", Scheduler::instance().clock(),session_->srcid(),frame_number , hdr_cmn::access(p)->size()-40,quant_,frametype_);

		

			fwrite(buf, strlen(buf), 1, BWFile);
			//printf("%-16f id %-16d udp %-16d\n", local_time, hdr_cmn::access(p)->frame_pkt_id_, hdr_cmn::access(p)->size()-28);
		}
		// add "beginning of talkspurt" labels (tcl/ex/test-rcvr.tcl)
		
		
		target_->recv(p);
	}
	n = nbytes % size_;
	if (n > 0) {
		p = allocpkt();
		double local_time = Scheduler::instance().clock();
		hdr_cmn::access(p)->size() = n;
		hdr_cmn::access(p)->frametype_=frametype_;
		hdr_cmn::access(p)->frame_number = frame_number;
		hdr_cmn::access(p)->quant = quant_;
		hdr_rtp* rh = hdr_rtp::access(p);
		rh->srcid() = session_->srcid();
		set_pkttype(PT_VIDEO);
		rh->flags() = 0;
		rh->seqno() = ++seqno_;
		rh->timestamp()=local_time;
		if (session_) {
 			session_->localsrc_update(size_);
 			
		}
		rh->srcid() = session_ ? session_->srcid() : 0;
		
		
		hdr_cmn::access(p)->timestamp() = 
		    (u_int32_t)(SAMPLERATE*local_time);
		hdr_cmn::access(p)->sendtime_ = local_time;	// (smallko)
		if(openfile!=0){
			hdr_cmn::access(p)->frame_pkt_id_ = id_++;

  			sprintf(buf, "%-16f %d %d %-16d %-8d %-8d \n", Scheduler::instance().clock(),session_->srcid(),frame_number , hdr_cmn::access(p)->size()-40,quant_,frametype_);
		


			fwrite(buf, strlen(buf), 1, BWFile); 
			//printf("%-16f id %-16d udp %-16d\n", local_time, hdr_cmn::access(p)->frame_pkt_id_, hdr_cmn::access(p)->size()-28);
		}
		// add "beginning of talkspurt" labels (tcl/ex/test-rcvr.tcl)
		
		
		target_->recv(p);
	}
	idle();

}

         


void RTPAgent::recv(Packet* p, Handler*)
{
	if (session_)
		session_->recv(p, 0);
	else
		Packet::free(p);
}

/** 
* to handle TCL commands 
*
*/
int RTPAgent::command(int argc, const char*const* argv)
{
	  if (argc == 4) {
		if (strcmp(argv[1], "send_nbytes") == 0) {
                        sendmsg(atoi(argv[2]),atoi(argv[3]));
                        return (TCL_OK);
                } 

	}
	 if (argc == 3) {
		if (strcmp(argv[1], "session") == 0) {
			session_ = (RTPSession*)TclObject::lookup(argv[2]);
			return (TCL_OK);
		}  
		
		if (strcmp(argv[1], "set_filename") == 0) {
			strcpy(BWfile, argv[2]);
			BWFile = fopen(BWfile, "w");
			openfile=1;
			return (TCL_OK);
		} 
		
		if (strcmp(argv[1], "quant") == 0) {
                        set_quant(atoi(argv[2]));
                        return (TCL_OK);
                }
		if (strcmp(argv[1], "prio") == 0) {
                        set_prio(atoi(argv[2]));
                        return (TCL_OK);
                }
		if (strcmp(argv[1], "frame_type") == 0) {
                        set_frametype(atoi(argv[2]));
                        return (TCL_OK);
                }
		
		
		
	}
	if(argc ==2) {		//added by smallko
		if (strcmp(argv[1], "closefile") == 0) {
			if(openfile==1)
				fclose(BWFile);
			return (TCL_OK);
		}
	
	} 

	
	return (Agent::command(argc, argv));
}







