/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1997 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the MASH Research
 * 	Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /cvsroot/nsnam/ns-2/apps/rtp.h,v 1.14 2000/09/01 03:04:06 haoboy Exp $
 */

#ifndef ns_rtp_h
#define ns_rtp_h

#include "config.h"
#include "object.h"
#include "agent.h"
#include "timer-handler.h"
#include "assp-agent.h"
#include "assp-sink.h"
#include <list>	

#define RTP_M 0x0080 // marker for significant events

class RTPReceiver;
class ASSPAgent;
class ASSPSink;


/** extensions to RTCP Sender Report */
struct sr_extensions {
	/** the id of the receiver */
        u_int32_t rcv_id_;
	/** the effective RTT measured by the sender */
	double eff_rtt_;
	u_int32_t& rcv_id() { return (rcv_id_); }
	double& eff_rtt() {return eff_rtt_;}	
	sr_extensions* next;

};

/** rtcp sender reports */
struct sender_report {
	
	/** the average low transmission rate of the low capacity stream */
	double low_tx_;
	/** the average transmission rate of the medium capacity stream */
	double medium_tx_;
	/** the average transmission rate of the high capacity stream */
	double high_tx_;
	/* per-field member functions */
	double octets_;
	u_int32_t T_epoch_;
	/** return the total RTP packets sent since the beginning of the session */
	
	double& octets_sent() {return octets_;}
	sr_extensions* s_ext_;
}; 

/** rtcp receiver reports */
struct receiver_report {
	
	/** the TCP friendly bandwidth share */
        double R_tcp_;
	/** last Sender Report time */
        double LSR_;
	/** delay since last sender report */
        double DLSR_;
	/** the delay jitter after filtering */
	double jitter_;
	/** flag for bye event */
	int  bye_;
	
	/* per-field member functions */
	
        /** return the the TCP friendly bandwidth share */
	double& R_tcp() { return (R_tcp_); }
	/** return the last Sender Report time */
	double& LSR() { return (LSR_); }
	/** return delay since last sender report */
	double& DLSR() { return (DLSR_); }
	/** return the the delay jitter after filtering */
	double& jitter() { return (jitter_); }
        int& bye() { return (bye_); }
	

}; 


/** rtp packet header */
struct hdr_rtp {
	/** the unique source id for this session */
	u_int32_t srcid_;
	/** sequence number of RTP and RTCP packets */
	int seqno_;
	/**rtp flags indicating significant event(begining of talkspurt) */
	u_int16_t flags_; 
	/** timestamp of the RTP packet */
	double timestamp_;
	
	static int offset_;
	inline static int& offset() { return offset_; }
	inline static hdr_rtp* access(const Packet* p) {
		return (hdr_rtp*) p->access(offset_);
	}
	/* per-field member functions */
	/** return the the unique source id for this session */
	u_int32_t& srcid() { return (srcid_); }
	/** return the sequence number of RTP and RTCP packets */
	int& seqno() { return (seqno_); }
	u_int16_t& flags() { return (flags_); }
	/** return the timestamp of the RTP packet */
	double& timestamp() { return (timestamp_); }
	
	sender_report* sr_;
	receiver_report* rr_;
	 
	
};

class RTPSource : public TclObject {
public:
	RTPSource* next;

	RTPSource(u_int32_t srcid);
	inline u_int32_t srcid() { return (srcid_); }
	inline int np() { return (np_); }
	inline int snp() { return (snp_); }
	inline int ehsr() { return (ehsr_); }
	/** returns the time of the Last Sender report */
	inline double LSR() { return (LSR_);  }
	/** returns the sender report time */
	inline double SRT() { return (SRT_);  }
	inline int nbytes() { return (nbytes_);  }
	

	inline void np(int n) { np_ += n; }
	inline void snp(int n) { snp_ = n; }
	inline void ehsr(int n) { ehsr_ = n; }
	/** the time of the Last Sender report */
	inline void LSR(double n) { LSR_= n; }
	/** the sender report time */
	inline void SRT(double n) { SRT_= n; }
	inline void nbytes(int n) {nbytes_ +=n;}
	
	
protected:
	u_int32_t srcid_;
	int np_;
	int snp_;
	int ehsr_;
	/** the time of the Last Sender report */
	double LSR_;
	/** the sender report time */
	double SRT_;
	int nbytes_;
	
};

/** this class holds the fields of the receivers in the RTP session.
   A receiver can only receive RTP packets and it is not a source
*/
class RTPReceiver : public TclObject {
public:
	/** instance of the RTPReceiver class */
        RTPReceiver* next;
	/** constructor */
	RTPReceiver(u_int32_t srcid);
	/** the source id of the RTPReceiver */
	inline u_int32_t srcid() { return (srcid_); }
	/** returns the effective RTT time measured by the sender */
	inline double eff_rtt() { return (eff_rtt_); }
	/** returns the sendind rate in bytes/sec */
	inline double rate() { return (rate_); }
	/** the effective RTT as it is measured by the RTP sender */
	inline void eff_rtt(double n) { eff_rtt_ = n; }
	  /** the TCP-friendly as it is reported by thr RTP receiver  */
	inline void rate(double n) { rate_ = n; }
	
            
protected:
	/** the srcid of this receiver */
	u_int32_t srcid_;
	/** the effective RTT as it is measured by the RTP sender */
	double eff_rtt_;
	/** the estimated trannsmission rate */
	double rate_;
	
};

class RTPSession : public NsObject {
public:
	RTPSession();
	~RTPSession();
	virtual void recv(Packet* p, Handler*);
	virtual void recv_ctrl(Packet* p);
	int command(int argc, const char*const* argv);
	inline u_int32_t srcid() { return (localsrc_->srcid()); }
	int build_report(int bye);
	void localsrc_update(int);
	//simulcast extensions
	RTPSession* next;
	void set_frametype(int fr_type);
	void send(int nbytes, int frame_number);
	void set_quant(int quant);
	/* access the RTP header */
	hdr_rtp* access_hdr_rtp();
	void set_join_flag(int a);
	void set_initial_rate(double value);
	inline int get_quality_level() {return quality_level_; }
protected:
	RTPSource* allsrcs_;
	RTPSource* localsrc_;
	int build_sdes();
	int build_bye();
	RTPSource* lookup(u_int32_t);
	void enter(RTPSource*);
	int last_np_;
	//simulcast extensions
	RTPReceiver* receivers_;
	ASSPAgent* assp_agent_;
	ASSPSink*  assp_sink_;
	RTPReceiver* lookup_rcv(u_int32_t);
	/* add this receiver */
	void enter_rcv(RTPReceiver*);
	/* removes the RTP receiver from the list */
	void remove_receiver(RTPReceiver*);
	/* removes the RTP sender from the list */
	void remove_sender(RTPSource*);

private:
	// fields
	/** the rtp header */
	hdr_rtp* rh;
	int enableFlowControl;
	/** the jitter delay as is it measured by the RTP receiver */
	double jitter;
	/** the smooth transmission rate */
	double smooth_rate;
	/** the RTT time */
	double RTT;
	/** weights for the smooth loss raton calculation */
	double weight[8];
	/** the array to keep the packet loss history */
	double pkt_loss_history[8];
	/** the smooth loss ratio */
	double smooth_loss;
	/** keeps the highest expected RTP sequence number */
	int last_ehsr;
	/** the transmission rate that is measured by the receiver */
	double tx_rate;
	/** the value A */
	double alpha;
	/** defines the responsiveness of the protocol */
	double smooth_factor;
	double ts_p, ts_n, tr_p, tr_n;
	double cjitter;
	int nlong_samples, nshort_samples;
	double long_avg_cjitter, short_avg_cjitter; 
	double long_sum, short_sum;
	double interframe_jitter;
	double T_one_way;
	double avg_rate;
	int we_sent;
	int isSender;
	int samples;
	list<double> lst;
	int join_flag;
	int quality_level_;
	
	

	//functions
	void sum_cjitter(double value);
	void avg_cjitter();
	void calculate_alpha(double value);
	void calculate_RTT();
	void increase_rate(int ps);
	void calculate_smooth_rate(double inst_tx);
	void calculate_smooth_loss (double fraction);
	void calculateR_tcp(int ps);
	void calculate_avg_rate(double rate);
	void update_rate();
	void resume();
	
};

class RTPAgent : public Agent {
 public:
        RTPAgent();
       
        virtual void recv(Packet* p, Handler*);
        virtual int command(int argc, const char*const* argv);
        void sendmsg(int nbytes, int frame_number);
 protected:
       
        
        RTPSession* session_;
        double lastpkttime_;
        int seqno_;
        int running_;
        
	int id_;	
	char BWfile[100];
	FILE *BWFile;
	int openfile;
	
        
      
       
};

#endif

