/** C++ Interface: simstream
//
// Description: 
//
//
// Author: Georgios Kioumourtzis <gkioumou@gmail.com>, (C) 2008
//

 * Copyright (c) 2007 University of Patras Computer Engineering and Informatics Department Greece
 *
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include "assp-agent.h"


static class ASSPAgentClass : public TclClass {
public:
	ASSPAgentClass() : TclClass("ASSP/Agent") {}
	TclObject* create(int, const char*const*) {
		return (new ASSPAgent());
	}
} class_assp_agent;

/**
 * no argument constructor 
 */
ASSPAgent::ASSPAgent()
	: sessions_(0),all_vbrs_(0),join_timer_(this),streamid_(-1)
{
	bind("low_tx", &low_tx);
	bind("medium_tx", &medium_tx);
	bind("high_tx", &high_tx);
	bind("timeout",&timeout);
	
	
	
	
}

/**
 * destructor
*/
ASSPAgent::~ASSPAgent()
{
	while (sessions_ != 0) {
		RTPSession* s = sessions_;
		sessions_ = sessions_->next;
		delete s;
	}
	
	
	
}

/**
 *  enters a session
 * @param s  the session to be added
 */
void ASSPAgent::enter(RTPSession* s)
{
	
	printf("Time:%f ASSP/Agent:%d ...I add session %d \n", NOW,streamid_ , s->srcid());
	s->next = sessions_;
	sessions_ = s;
	
}


/**
  * the command function that handles TCL commands
 *
 */
int ASSPAgent::command(int argc, const char*const* argv)
{
	if (argc == 3) {
		if (strcmp(argv[1], "session") == 0) {
			RTPSession* s = (RTPSession*)TclObject::lookup(argv[2]);
			enter(s);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "stream-id") == 0) {
			streamid_ = atoi(argv[2]);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "attach-vbr") == 0) {
			vbr_rateadapt2* s = (vbr_rateadapt2*)TclObject::lookup(argv[2]);
			enter(s);
			return (TCL_OK);
		}
		
	}
	if (argc == 2) {
		if (strcmp(argv[1], "start-timer") == 0) {
			start_timer(timeout);
			return (TCL_OK);
		}
	


	}

	return (TclObject::command(argc, argv));
}

void ASSPAgent::enter(vbr_rateadapt2* vbr)
{
	printf("Time:%f ASSP/Agent:%d ...I add vbr  \n", NOW,streamid_ );
	vbr->next = all_vbrs_;
	all_vbrs_ = vbr;
}

void ASSPAgent::set_frametype(u_int32_t q_level, int fr_type)
{
	for (RTPSession* s = sessions_; s != 0; s = s->next) {
		if(q_level == (u_int32_t)s->get_quality_level()) {
			
			s->set_frametype(fr_type);
			break;
		}
	}
	
}

void ASSPAgent::send(u_int32_t q_level, int nbytes, int frame_number)
{
	for (RTPSession* s = sessions_; s != 0; s = s->next) {
		if(q_level == (u_int32_t)s->get_quality_level()) {
//		printf(" ASSPAgent::send TIME %f q_level %d s->srcid() %d \n", NOW, q_level, s->srcid());
			s->send(nbytes,frame_number);
			break;
		}
	}
}
	

void ASSPAgent::set_quant(u_int32_t q_level, int quant)
{
	for (RTPSession* s = sessions_; s != 0; s = s->next) {
		if(q_level == (u_int32_t)s->get_quality_level()) {
			s->set_quant(quant);
			break;
		}
	}
}


void ASSPAgent::set_new_tx_rate(int quality_level, double newRate)
{
	// new rate in bytes/s
	if(quality_level == 0) {
		vbr_rateadapt2 *vbr = lookup(quality_level);
	//	printf("vbr %d \n", p->get_qlevel());
		
		vbr->adapt_rate(newRate);
	} 
	else if(quality_level == 1) {
		vbr_rateadapt2 *vbr = lookup(quality_level);
	//	printf("vbr %d \n", p->get_qlevel());
		
		vbr->adapt_rate(newRate);
	} 
	else  {
		vbr_rateadapt2 *vbr = lookup(quality_level);
	//	printf("vbr %d \n", p->get_qlevel());
		
		vbr->adapt_rate(newRate);
	}
	
}

vbr_rateadapt2* ASSPAgent::lookup(u_int32_t q_level )
{
	vbr_rateadapt2 *p;
	for (p = all_vbrs_; p != 0; p = p->next)
		if (p->get_qlevel() == q_level)
			return (p);

	return (0);
}

void ASSPAgent::set_Avg_tx(int quality_level, double rate)
{
	switch(quality_level) {
		case 0	:
			low_tx = rate;
		//	printf("ASSPAgent::set_Avg_tx() TIME %f low %f \n", NOW, low_tx);
			break;
		case 1	:
		//	printf("ASSPAgent::set_Avg_tx() TIME %f medium %f \n", NOW, medium_tx);
			medium_tx = rate;
			
			break;
		case 2	:
		//	printf("ASSPAgent::set_Avg_tx() TIME %f high %f \n", NOW, high_tx);
			high_tx = rate;
			break;
		default	:	
		printf("Error...no session found\n");
	}
	
//	printf("ASSPAgent::set_Avg_tx TIME %f Stream_id %d low %f medium %f high %f \n", NOW, streamid_, low_tx, medium_tx, high_tx);
	
	
}

void JoinTimer::expire(Event* /*e*/) {
        a_->join_timeout(0);
}

void ASSPAgent::start_timer(int a)
{
	join_timer_.resched(a);
}

void ASSPAgent::join_timeout(int )
{
//	printf("ASSPAgent::join_timeout TIME %f \n", NOW);
	for (RTPSession* s = sessions_; s != 0; s = s->next) {
		s->set_join_flag(1);
	}
	join_timer_.resched(timeout);
	
}

double ASSPAgent::get_low()
{
	return low_tx;
}

double ASSPAgent::get_medium()
{
	return medium_tx;
}

double ASSPAgent::get_high()
{
	return high_tx;
}





