/** C++ Interface: simstream
//
// Description: 
//
//
// Author: Georgios Kioumourtzis <gkioumou@gmail.com>, (C) 2008
//

 * Copyright (c) 2007 University of Patras Computer Engineering and Informatics Department Greece
 *
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include "assp-sink.h"

#define ALPHA 0.3
#define UP_LEVEL_FACTOR 1.2
#define DOWN_LEVEL_FACTOR 0.8
#define RATE_FACTOR 0.7

static class ASSPSinkClass : public TclClass {
public:
	ASSPSinkClass() : TclClass("ASSP/Sink") {}
	TclObject* create(int, const char*const*) {
		return (new ASSPSink());
	}
} class_assp_sink;

/**
 * no argument constructor 
 */
ASSPSink::ASSPSink()
	: sessions_(0),streamid_(-1)
{
	seqno_ = -1;
	id_ = 0;
	openfile = 0;
	expected_ = 0;
	bind("low_tx", &low_tx);
	bind("medium_tx",&medium_tx);
	bind("high_tx",&high_tx);
	bind("level_",&level_);
	bind("rcv_bytes",&rcv_bytes);
	bind("low_threshold", &low_threshold);
	bind("medium_threshold", &medium_threshold);
	bind("high_threshold", &high_threshold);
	rx_rate = 0;
	
}

/**
 * destructor
*/
ASSPSink::~ASSPSink()
{
	while (sessions_ != 0) {
		RTPSession* s = sessions_;
		sessions_ = sessions_->next;
		delete s;
	}
	
	
	
}


/**
 *  enters a session
 * @param s  the session to be added
 */
void ASSPSink::enter(RTPSession* s)
{
	
//	printf("Time:%f ASSP/Sink:%d ...I add session %d \n", NOW,streamid_ , s->srcid());
	s->next = sessions_;
	sessions_ = s;
	
}

void ASSPSink::rcv(Packet* p)
{
	
	/*** Evalvid-RA: Added by A. Lie 020206 ***/
	hdr_rtp* rh = hdr_rtp::access(p); 
	hdr_cmn* hdr=hdr_cmn::access(p);	
	seqno_ = hdr->frame_pkt_id_;	
	int frame_number = hdr->frame_number;
	int quant = hdr->quant;
	int srcid = rh->srcid_;
//	seqno_ = rh->seqno_;
//	printf("Time:%f ASSP/Sink:%d ...I receive from Session %d \n", NOW,streamid_ , rh->srcid());
	if (expected_ >= 0 && openfile==1 ) {
	//	int loss = seqno_ - expected_;
// 		if (loss > 0) {
// 			while (loss>0){
// 				fprintf(BWFile,"%-16f id %-16d lost %-16d\n", 
// 						  Scheduler::instance().clock(), id_++, 9999);	
// 				loss-=1;
// 			}
// 		}
		//	    } else {
//  		if (id_ != seqno_) {
//  			printf("Something is not correct, Rx/Tx non-synch, rx_id=%d, tx_id=%d\n", id_, seqno_);
// 			} else
			fprintf(BWFile,"%-16f %d  %d   %d   %-16d\n", 
				  Scheduler::instance().clock(), srcid,frame_number , hdr->size()-40,quant);
		rcv_bytes+= hdr->size();
		// }
	}	
	expected_ = seqno_ + 1;
	Packet::free(p); 
  
}


/**
  * the command function that handles TCL commands
 *
 */
int ASSPSink::command(int argc, const char*const* argv)
{
	if (argc == 3) {
		if (strcmp(argv[1], "session") == 0) {
			RTPSession* s = (RTPSession*)TclObject::lookup(argv[2]);
			enter(s);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "stream-id") == 0) {
			streamid_ = atoi(argv[2]);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "set_filename") == 0) {
			strcpy(BWfile, argv[2]);
			BWFile = fopen(BWfile, "w");
			openfile=1;
			return (TCL_OK);
		} 
		if (strcmp(argv[1], "set_initial_rate") == 0) {
			set_int_rate(atoi(argv[2]));
			return (TCL_OK);
		} 
		
	}
	if(argc ==2) {		//added by smallko
		if (strcmp(argv[1], "closefile") == 0) {
			if(openfile==1)
				fclose(BWFile);
			return (TCL_OK);
		}
	} 

	return (TclObject::command(argc, argv));
}

void ASSPSink::setLow_tx(double value)
{
	low_tx = value;
}

void ASSPSink::setMedium_tx(double value)
{
	medium_tx = value;
}

void ASSPSink::setHigh_tx(double value)
{
	high_tx = value;
	
}

void ASSPSink::setInst_rx_rate(double value)
{
	rx_rate = rx_rate * (1-ALPHA) + value * ALPHA;
//	rx_rate = rx_rate * 0.7 + value * 0.3;
//	printf("ASSPSink::setInst_rx_rate() Time:%f Sink_id:%d rx_rate %f \n", NOW,streamid_ , rx_rate);
}




void ASSPSink::compare()
{
/*	printf("ASSPSink::compare() Time:%f Stream:%d  Avg_rx: %f Avg_Tx_Low:%f Avg_Tx_Medium:%f Avg_Tx_High:%f \n", NOW,streamid_ , rx_rate * 8, low_tx * 8, medium_tx * 8, high_tx * 8  );	*/

	switch(level_) {
		case 0	:

			if(rx_rate >= UP_LEVEL_FACTOR * medium_threshold && rx_rate >= medium_tx * RATE_FACTOR) {
			 go_up(level_);
			}
			break;
		case 1	:
	
			if(rx_rate <= DOWN_LEVEL_FACTOR * medium_threshold) {

				go_down(level_);
			} else if(rx_rate >= UP_LEVEL_FACTOR * high_threshold && rx_rate >= high_tx * RATE_FACTOR) {

				go_up(level_);
			}
			break;
		case 2	:
	
			if(rx_rate <= DOWN_LEVEL_FACTOR * high_threshold ) {

			go_down(level_);
			}
			break;
		default	:	
		printf("Error...no level found\n");
	}
}

/**
 *  go up one level
 * @param level  the current level
 */
void ASSPSink::go_up(int level)
{
	printf("Time:%f Stream:%d level:%d I go UP  Avg_rx: %f Avg_Tx_Low:%f Avg_Tx_Medium:%f Avg_Tx_High:%f \n", NOW,streamid_ , level, rx_rate * 8, low_tx * 8, medium_tx * 8, high_tx * 8  );
	Tcl::instance().evalf("%s go-up %d",name(),level);
	
	
}

/**
 * go down one level
 * @param level  the current level
 */
void ASSPSink::go_down(int level)
{
	printf("Time:%f Stream:%d level:%d I go DOWN  Avg_rx: %f Avg_Tx_Low:%f Avg_Tx_Medium:%f Avg_Tx_High:%f \n", NOW,streamid_ , level, rx_rate * 8, low_tx * 8, medium_tx * 8, high_tx * 8  );
	Tcl::instance().evalf("%s go-down %d",name(),level);
	
}

void ASSPSink::set_int_rate(double value)
{
	for (RTPSession* s = sessions_; s != 0; s = s->next) {
		s->set_initial_rate(value);
	}
}




