/** C++ Interface: vbr_rateadapt
//
// Description: 
//
//
// Author: Georgios Kioumourtzis <gkioumou@gmail.com>, (C) 2008
// Based on: mytraffictrace2.cc made by Ke ChihHeng, Taiwan
// and Arne Lie, NTNU Dept. of Telematics, Trondheim, Norway   
 * Copyright (c) 2007 University of Patras Computer Engineering and Informatics Department Greece
 *
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef ns_vbr_rateadapt_h
#define ns_vbr_rateadapt_h

#include <sys/types.h>
#include <sys/stat.h> 
#include <stdio.h>

#include "config.h"
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif

#include "random.h"
#include "object.h" 
#include "trafgen.h"
#include "assp-agent.h"

#define MAX_RA_VARS 30 /* Maximum number of rate variations */
#define MAX_NODE 50  /* Maximum number of P-AQM routers sending ECF to this source */

class ASSPAgent;


/**The following struct is used to hold all data about the transmitted video sequence */
struct tracerec {
	u_int32_t trec_time; /* inter-packet time (usec) */
	u_int32_t trec_size[MAX_RA_VARS]; /* frame size (bytes). AL: store all versions! */
	u_int32_t trec_type; /* packet type */ 
	u_int32_t trec_max; /* maximun fragmented size (bytes) */
	u_int32_t trec_source; /* AL: source number (i.e. "pointer" to which version of 
									  video that is used for this frame) */
};


/** The class to open and read the trace files */

class vbrTraceFile : public NsObject {
public:
	vbrTraceFile();
	void get_next(int&, struct tracerec&, int Q, int flag); /* called by TrafficGenerator
																				* to get next record in trace.
																				*/
	int setup();  /* initialize the trace file */ 
	int command(int argc, const char*const* argv);
	int get_a();		//added by smallko
	int num_extra_RA;
	int status_; 
	
private:
	void recv(Packet*, Handler*); /* must be defined for NsObject */
	char *name_;  /* name of the file in which the binary trace is stored */
	int nrec_;    /* number of records in the trace file */
	struct tracerec *trace_; /* array holding the trace (AL: of all rate variants!) */
	int a_;
};



/** instance of a traffic generator.  has a pointer to the TraceFile
 * object and implements the interval() function.
 */

class vbr_rateadapt2 : public TrafficGenerator {
public:
	vbr_rateadapt2();
	int command(int argc, const char*const* argv);
	virtual double next_interval(int &);
	vbr_rateadapt2* next;
	inline u_int32_t get_qlevel() { return (q_level_); }
	void adapt_rate(double rate);

	
	
protected:
	void timeout();
	virtual void start();
	virtual void stop(); 
	int running2_;
	vbrTraceFile *tfile_;
	struct tracerec trec_;
	int ndx_;
	int f_, max_; // added by smallko
	int source_; // Added by AL
	int Q; // Added by AL 311005
	int Q_wait; // Added by AL 071205 to test imidiate Q-change, but not until current frame
	// is finished transmitted
	int Q_flag;
	int R_hist[20]; // store up to last 20 frame sizes
	int frameIndex;
	double r_, b_, q_; // The Hamdi LB(r,b) and q parameters
	double GoP_, fps_; // Needed to scale the values correctly
	double lb_X; // The Leaky Bucket X(k) value
	double lb_R; // The monitored R(ate) of the last GOP
	void init();
	int is_mine; // AL: added to differ between the investigated video, and the simple tracers
	/* The following variables are for ECF rate adaptivity */
	double ecf_new;
	int from_f;
	double new_rate, new_b;
	double ecf_store[MAX_NODE];  /* can handle 10 P-AQM nodes */
	double rate_store[MAX_NODE]; /* ---------  "  ----------- */
	double ECF_rateadapt[MAX_NODE];
	double rate_;   /* send rate during on time (bps) */
	int numbFrames;
	double old_rate_coeff, new_rate_coeff;
	double b_factor;
	int whoami_i;
	int numbGops;
	int update_count;
	int sub_sample;
	//ASSP
	ASSPAgent* assp_agent_;
	u_int32_t q_level_;
	int Q_low, Q_high;
	int frame_number;
	
	
	
};

#endif
