/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1997 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the MASH Research
 * 	Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char rcsid[] =
    "@(#) $Header: /cvsroot/nsnam/ns-2/common/session-rtp.cc,v 1.13 2000/09/01 03:04:07 haoboy Exp $";
#endif

#include <stdlib.h>
#include "packet.h"
#include "ip.h"
#include "rtp.h"
#include <math.h>

// Default values.. you can change them for your experiments
 #define CONGESTED 0.2
 #define UNLOADED  0.9
#define PKTSIZE 1012
 

#define NOW Scheduler::instance().clock()



static class RTPSourceClass : public TclClass {
public:
	RTPSourceClass() : TclClass("RTPSource") {}
	TclObject* create(int argc, const char*const* argv) {
		if (argc >= 5)
			return (new RTPSource(atoi(argv[4])));

		return 0;
	}
} class_rtp_source;

static class RTPSessionClass : public TclClass {
public:
	RTPSessionClass() : TclClass("Session/RTP") {}
	TclObject* create(int, const char*const*) {
		return (new RTPSession());
	}
} class_rtp_session;

RTPSession::RTPSession() 
	: allsrcs_(0), localsrc_(0), last_np_(0),receivers_(0),assp_agent_(0),assp_sink_(0)
{
	bind("enableFlowControl", &enableFlowControl);
	bind("cjitter", &cjitter);
	bind_bool("isSender",&isSender);
	
	
	rh = 0;
	jitter = 0;
	smooth_rate = 0;
	RTT = 0;
	smooth_loss = 0;
	last_ehsr = 0;
	tx_rate = 0;
	alpha = 1;
	smooth_factor = 0.1;
	ts_p= ts_n= tr_p= tr_n=0;
	nlong_samples =  nshort_samples = 0;
	long_avg_cjitter =  short_avg_cjitter = 0; 
	long_sum =  short_sum = 0;
	interframe_jitter = 0;
	T_one_way = 0;
	weight[0] = 1;
	weight[1] = 1;
	weight[2] = 1;
	weight[3] = 1;
	weight[4] = 0.8;
	weight[5] = 0.6;
	weight[6] = 0.4;
	weight[7] = 0.2;
	avg_rate = 0;
	we_sent = 0;
	samples = 0;
	join_flag = 0;
	quality_level_ = -1;
	
}

RTPSession::~RTPSession() 
{
	while (allsrcs_ != 0) {
		RTPSource* p = allsrcs_;
		allsrcs_ = allsrcs_->next;
		delete p;
	}
	delete localsrc_;
	while (receivers_ != 0) {
                RTPReceiver* p = receivers_;
                receivers_ = receivers_->next;
                delete p;
	}
	lst.~list<double>();
}

void RTPSession::localsrc_update(int size)
{
	localsrc_->np(1);
	localsrc_->nbytes(size);
}

#define RTCP_HDRSIZE 8
#define RTCP_SR_SIZE 20
#define RTCP_RR_SIZE 48

/**
 *  the method for creating the SR and RR reports
 *  the "heart" of the RTCP protocol
 * @param bye if there is a bye event
 * @return the length of this report in bytes
 */
int RTPSession::build_report(int bye)
{
//printf("RTPSession::build_report() TIME %f local %d \n", NOW, localsrc_->srcid());
	rh = new hdr_rtp;
	rh->rr_ = 0;
        rh->sr_ = 0;
	
        int nsrc = 0;
        int nrr = 0;
        int len = RTCP_HDRSIZE;
        we_sent = 0;
	double fraction = 0.0;
	
        if (localsrc_->np() != last_np_) {
                last_np_ = localsrc_->np();
                we_sent = 1;
                len += RTCP_SR_SIZE;
		//add sender report
                sender_report* sr;
                //fill the report
                sr = new sender_report;
		sr->s_ext_ = 0;
                sr->octets_sent() = localsrc_->nbytes();
		if(join_flag == 1) {
			sr->T_epoch_ = 1;
			join_flag = 0;
		} 
		else  sr->T_epoch_ = 0;
		sr->low_tx_ = assp_agent_->get_low();
		sr->medium_tx_ =  assp_agent_->get_medium();
		sr->high_tx_ = assp_agent_->get_high();
		
		RTPReceiver *p;
        	for (p = receivers_; p != 0; p = p->next) {
        		sr_extensions* se = new sr_extensions;
			se->rcv_id()=p->srcid();
			se->eff_rtt()=p->eff_rtt();
			se->next = sr->s_ext_;
			sr->s_ext_ = se;
        	}
	        //store the report
                rh->sr_ = sr;
//		assp_agent_->set_Avg_tx(localsrc_->srcid(), avg_rate);
//		printf("TIME %f local %d avg_rate %f \n", NOW, localsrc_->srcid(),avg_rate);
//		resume();

        }
        for (RTPSource* sp = allsrcs_; sp != 0; sp = sp->next) {
                ++nsrc;
                int received = sp->np() - sp->snp();
		
                if (received == 0) {

                        continue;
                }
		if(localsrc_->srcid() != sp->srcid()) {
		
//		printf("RTPSession::build_report() TIME %f local %d \n", NOW, localsrc_->srcid());

                sp->snp(sp->np());
                len += RTCP_RR_SIZE;
		// calculate loss fraction since previous report
		int expected_interval  = sp->ehsr() - last_ehsr;
		last_ehsr = sp->ehsr();
		int lost_interval = expected_interval -  received;


		if (lost_interval <= 0 || expected_interval == 0 ) {
			 fraction = 0;

		} else fraction = ((double)lost_interval  / (double)expected_interval);
               
		// Update the R_tcp according to fraction value
		if( sp->np() >= 1) {// I have already received RTP packets from the source
			if(fraction == 0) {
				increase_rate(PKTSIZE);

			} 	else /*if(fraction != 0)*/ {
					calculate_smooth_loss(fraction);
					calculateR_tcp(PKTSIZE);
				}
		}
		//add receiver report
                receiver_report* rr;
                rr = new receiver_report;
                //fill the report
		
		rr->LSR() = sp->LSR();
		rr->DLSR()= NOW - sp->SRT();
		rr->R_tcp() = smooth_rate;

		avg_cjitter();
	//	printf("TIME %f local %d rate %f \n", NOW, localsrc_->srcid(), 8 * avg_rx_rate);
//		assp_sink_->setAvg_rx(avg_rate);
//		resume();
		
		
		
		if(bye) {
		rr->bye()= 1;
		remove_sender(sp);
		} else rr->bye() = 0;
		
		rh->rr_ = rr;
	//	printf("TIME %f local %d \n", NOW, localsrc_->srcid());

                if (++nrr >= 31)
                        break;
		} // end if

        }// end for loop


        if (bye) {
                len += build_bye();
		}
        else
                len += build_sdes();

        Tcl::instance().evalf("%s adapt-timer %d %d %d", name(),
                              nsrc, nrr, we_sent);
        Tcl::instance().evalf("%s sample-size %d", name(), len);

        return (len);
}


int RTPSession::build_bye() 
{
	return (8);
}

int RTPSession::build_sdes()
{
	/* XXX We'll get to this later... */
	return (20);
}

void RTPSession::resume() 
{
	avg_rate = 0;
	samples = 0;
}


/**
 * accesses the RTP packet header
 * @return , the
 */
hdr_rtp* RTPSession::access_hdr_rtp()
{
	return rh;
}



void RTPSession::recv(Packet* p, Handler*)
{
	hdr_cmn* mh = hdr_cmn::access(p);
	hdr_rtp* rh = hdr_rtp::access(p);
       	RTPSource* s = lookup(rh->srcid());

    	if (s == 0) {
                if(rh->srcid()!=localsrc_->srcid()){
                        Tcl& tcl = Tcl::instance();
                        tcl.evalf("%s new-source %d", name(),rh->srcid());
                        s = (RTPSource*)TclObject::lookup(tcl.result());
			s->ehsr(rh->seqno());
			
		}
        }
	if(rh->srcid()!=localsrc_->srcid())
	{
 		// calculate one way trip time
 		T_one_way = NOW - rh->timestamp();
		calculate_RTT();
		// calculate interframe jitter
		ts_p = ts_n;
		ts_n = rh->timestamp();
		tr_p = tr_n;
		tr_n = NOW;
		interframe_jitter = (tr_n-tr_p)-(ts_n-ts_p);
		// this is the cumulative jitter
		cjitter+= interframe_jitter;
// 		// we use it for the averaging functions
 		sum_cjitter(cjitter);
		
		s->np(1); // counts the packets that I have received from sender s
		s->ehsr(rh->seqno());// the first ehsr = -1
		s->nbytes(mh->size());
    		if (assp_sink_)
		assp_sink_->rcv(p);
	}
        Packet::free(p);

}



/**
 * We sum up the measured cumulative jitter and keep a moving long and short 
 * average. 
 * @param value  the new value of cumulative jitter
 */
void RTPSession::sum_cjitter(double value)
{
	nlong_samples++;
	nshort_samples++;
	long_avg_cjitter = long_avg_cjitter + ((value - long_avg_cjitter)/nlong_samples);
	short_sum += value;
//	printf("Time %f value %f samples %d AVG-CJITTER %f \n", NOW,value,nlong_samples, long_avg_cjitter );   

}

/**
 * We compare the long and short averages. When the short average
 * is bigger than the long average it is an indication of upcoming
 * congestion. The CONGESTED factor takes a low value to smooth
 * the transmission rate of the sender.When the network is not congested we
 * ask from the sender to increase the transmission rate.
 */
void RTPSession::avg_cjitter()
{
	if(nshort_samples >0) {	
	short_avg_cjitter = short_sum/(double)nshort_samples;
	}
	if(short_avg_cjitter >= long_avg_cjitter ) {
		smooth_factor = CONGESTED;
//	printf("TIME-CONGESTED %f localsrc_  %d  short_avg_cjitter %f  long_avg_cjitter %f \n", NOW, localsrc_->srcid(),short_avg_cjitter, long_avg_cjitter  );   
	} else {
		smooth_factor = UNLOADED;
	}
	
	nshort_samples = 0;
	short_sum = 0;
	
	
}

/**
 *  the function that calculate the A value
 * @param value the effective RTT that is measured by the RTP sender
 */
void RTPSession::calculate_alpha(double value)
{

	if( value == 0) {
		alpha = 1;
	} else
	alpha = (value/T_one_way) -1;
}

/**
 *  calculates the RTT time
 *  one_way is the one way trip time
 *  eff is the effective RTT time measured and reported by the sender of the
 *  RTP packets
 * @param eff, the effective RTT time measured by the sender 
 */
void RTPSession::calculate_RTT()
{
	double temp = (1+alpha) * T_one_way;	
	RTT = temp * 0.1 + 0.9 * RTT;   //taken from RFC 3448
	
//	printf("TIME %f RTT %f \n", NOW, RTT_);	
/* old calculations
 	double temp = (1+alpha) * T_one_way_;
 
	RTT_ = temp * 0.1 + 0.9 * RTT_;   //taken from RFC 3448
//	RTT_ = 0.875 * RTT_ + 0.125 * temp; // RFC 2988
*/

}

/**
 * the estimation made by the receiver of the transmission rate when
 *  fraction loss is zero. The increase is not higher than 1 RTP packet/current rate
 *  in bytes/sec
 *  sample_tx is the current estimation of the RTP receiver. We pass this value from
 *  the filter
 * @param ps, the packet size of the RTP packet
 */
void RTPSession::increase_rate(int ps)
{
	double sample_tx = smooth_rate + 4 * (double)ps/RTT;
	// pass sample_tx from the filter
	calculate_smooth_rate(sample_tx);
}
/**
 * this is the filter function for the smooth transmission rate measurement
 * @param inst_tx  the current transmission rate estimation by the RTP receiver
 */
void RTPSession::calculate_smooth_rate(double inst_tx)
{
   	smooth_rate = smooth_factor * inst_tx + smooth_rate * (1 - smooth_factor);
	
	assp_sink_->setInst_rx_rate(smooth_rate);
// 	if( localsrc_->srcid() == 6) {
// 	printf("RTPSession::calculate_smooth_rate() TIME: %f localsrc_ %d smooth_rate %f \n", NOW, localsrc_->srcid(),smooth_rate);
// 	}
}

void RTPSession::calculate_avg_rate(double rate)
{
	samples++;
	avg_rate = avg_rate + ((rate - avg_rate)/(double)samples);
	assp_agent_->set_Avg_tx(quality_level_, avg_rate);
//	printf("calculate_avg_rate() TIME %f localsrc_ %d avg_rate %f \n", NOW,localsrc_->srcid(), avg_rate);
	
	
}

/**
 * measure_smooth_loss is called when the last measured by the receiver
 * loss fraction is not zero.
 * @param fraction  the fraction of lost RTP packets/expected RTP packets
 * since last RR
 */
void RTPSession::calculate_smooth_loss (double fraction)
{
	
	double I_tot0 = 0;
	double I_tot1 = 0;
	double W_tot = 0;
	for (int i=0; i<7; i++) {
 	pkt_loss_history[i+1]= pkt_loss_history[i];
 	}
 	pkt_loss_history[0] = fraction;

	for (int i=0; i<7; i++) {
	I_tot0 = I_tot0+ (pkt_loss_history[i] * weight[i]);
	W_tot = W_tot +  weight[i];
	}

	for (int i=1; i<8; i++) {
	I_tot1 = I_tot1 + (pkt_loss_history[i] * weight[i-1]);
	}
	double I_tot =0;
	if( I_tot0 > I_tot1) {
	I_tot = I_tot0;
        } else {
	I_tot = I_tot1;
	}
	double  I_mean = I_tot/W_tot;

	smooth_loss= I_mean;
}

/**
 *  calculates the TCP friendly bandwidth share  
 * @param ps is the size of the RTP packet
 */
void RTPSession::calculateR_tcp(int ps)
{

	double min =0;
	double fraction = 3 * sqrt((3 * smooth_loss) / 8);
	if(fraction <  1 ){
		min = fraction;
	}	else min =1;
	// TCP friendly BW share equation
	double sample_tx = (double)ps / (RTT *  (sqrt(2 * smooth_loss /3)) + 4 * (RTT * min * smooth_loss) * (1 + (32 * pow(smooth_loss,2))));
	// pass sample_tx from the filter
	calculate_smooth_rate(sample_tx);
}

/**
 * the function that accesses and exploits the SR and RR
 *  here is where the sender calculates the effective RTT time based on 
 *  the receiver's RR. The sender also updates its transmission rate based 
 *  on the receivers' RR.
 *  We don't exploit the SR in this implementation but a third party eg
 *  a network controller can exploit both SR and RR, witout in fact joining the session
 * @param p the RTCP packet
 */
void RTPSession::recv_ctrl(Packet* p)
{
	hdr_cmn* mh = hdr_cmn::access(p);
	hdr_rtp* rh = hdr_rtp::access(p);
	u_int32_t local_src = localsrc_->srcid();
	
	if(rh->srcid() != local_src) {
	
		if (rh->sr_ != 0) { // get the Sender report
			RTPSource* source = lookup(rh->srcid());
			if (source != 0 && source->srcid()!= local_src) {
				source->LSR(rh->timestamp());
				source->SRT(NOW);
				assp_sink_->setLow_tx(rh->sr_->low_tx_);
				assp_sink_->setMedium_tx(rh->sr_->medium_tx_);
				assp_sink_->setHigh_tx(rh->sr_->high_tx_);
				if(rh->sr_->s_ext_ != 0) {
				sr_extensions* se = new sr_extensions;
				for (se = rh->sr_->s_ext_; se != 0; se = se->next) {

					if(se->rcv_id()== local_src) {
					
						if(se->eff_rtt() == 0) {
							calculate_alpha(0);
						} else calculate_alpha(se->eff_rtt());
					}
						
				}// end for
				}
				
			}

			if(rh->sr_->T_epoch_ == 1) assp_sink_->compare();
		
		}

		if(isSender){ // get the Receiver report

				if(rh->rr_ != 0){
					RTPReceiver* s = lookup_rcv(rh->srcid());
						
						if (s != 0) {
							for (s = receivers_; s!= 0; s = s->next) {
		
								if(rh->srcid() == s->srcid()) {
									
									double eff_rtt = 0.0;
								
									if(rh->rr_->LSR()!=0) {
									eff_rtt = NOW - rh->rr_->LSR() - rh->rr_->DLSR();
									}
 									s->eff_rtt(eff_rtt);
                                 					s->rate(rh->rr_->R_tcp());
									if(rh->rr_->bye() == 1) {
									     	remove_receiver(s);
									}
									
		                                                }	

							} // end for loop
							if(enableFlowControl == 1 && receivers_) {
								update_rate();
								} /*else {
									printf("TIME: %f Sender %d NOT HAVING RECEIVERS \n", NOW,localsrc_->srcid());
									
									
									}*/
						} //end if
						else {
							s = new RTPReceiver(rh->srcid());
							enter_rcv(s);
						}
						
					}
				}
	}

	Tcl::instance().evalf("%s sample-size %d", name(), mh->size());
	Packet::free(p);
}

void RTPSession::update_rate()
{
       RTPReceiver* s;
	for (s = receivers_; s!= 0; s = s->next) {
		if(s->rate() > 0) {
			lst.push_front(s->rate());
		}
	}
	lst.sort();
	double inst_tx =lst.front();
	lst.clear();
	if(inst_tx != tx_rate) {
	//	tx_rate_ = inst_tx;
			
		if(inst_tx <  tx_rate/2) {
			tx_rate = tx_rate/2;
		} else tx_rate = inst_tx;
	
	Tcl::instance().evalf("%s session_bw %fb/s",name(),8*tx_rate);
//	printf("RTPSession::update_rate() TIME: %f Local %d \n", NOW, localsrc_->srcid());
	if(assp_agent_)
		assp_agent_->set_new_tx_rate(quality_level_, tx_rate);
	
	}
	calculate_avg_rate(tx_rate);	
}




/**
 * remove a receiver from a sender's list
 * @param p the RTPReceiver to be removed
 */
void RTPSession::remove_receiver(RTPReceiver* p)
{

//        printf("Time:%f Sender:%d ...I remove receiver %d \n", NOW,localsrc_->srcid(),p->srcid());
	
	RTPReceiver* s;
	RTPReceiver* q;
        if (receivers_== p)
        {
            	q = receivers_;
                receivers_= receivers_->next;
                delete q;
        }
        else
        {
        	for (s = receivers_; s!=0; s=s->next)
        	{
        		if (s->next==p)
                	{
                 		q = s->next;
                 		s->next=q->next;
                 		delete q; 
                 	}	
                }
        }

}

/**
 *  we remove this sender from the list upon a bye event
 * @param p  the sender that has to be removed from our sources' list
 */
void RTPSession::remove_sender(RTPSource* p)
{
//	printf("Time:%f Receiver:%d ...I remove sender %d \n", NOW,localsrc_->srcid(), p->srcid());
	if(p->srcid() != localsrc_->srcid()) {
	RTPSource* s;
	RTPSource* q;
        if (allsrcs_==p)
        {
            	q = allsrcs_;
                allsrcs_= allsrcs_->next;
                delete q;
        }
        else
        {
        	for (s = allsrcs_; s!=0; s=s->next)
        	{
        		if (s->next==p)
                	{
                 		q = s->next;
                 		s->next=q->next;
                 		delete q; 
                 	}	
                }
        }	
	
	}
}




/* XXX Should hash this... */
RTPSource* RTPSession::lookup(u_int32_t srcid)
{
	RTPSource *p;
	for (p = allsrcs_; p != 0; p = p->next)
		if (p->srcid() == srcid)
			return (p);

	return (0);
}

void RTPSession::enter(RTPSource* s)
{
//	printf("Time:%f Receiver:%d ...I add sender %d \n", NOW,localsrc_->srcid(), s->srcid());
	s->next = allsrcs_;
	allsrcs_ = s;
}

int RTPSession::command(int argc, const char*const* argv)
{
	if (argc == 3) {
		if (strcmp(argv[1], "enter") == 0) {
			RTPSource* s = (RTPSource*)TclObject::lookup(argv[2]);
			enter(s);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "localsrc") == 0) {
			localsrc_ = (RTPSource*)TclObject::lookup(argv[2]);
			enter(localsrc_);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "assp-agent") == 0) {
			assp_agent_ = (ASSPAgent*)TclObject::lookup(argv[2]);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "assp-sink") == 0) {
			assp_sink_ = (ASSPSink*)TclObject::lookup(argv[2]);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "quality_level") == 0) {
			if (strcmp(argv[2], "low") == 0) {
				quality_level_ =0;
			}
			if (strcmp(argv[2], "medium") == 0) {
				quality_level_ = 1;
			}
			if (strcmp(argv[2], "high") == 0) {
				quality_level_ = 2;
			}
			
			printf("RTPSession::command quality_level_ %d \n", quality_level_);
			return (TCL_OK);
		}

	}
	return (TclObject::command(argc, argv));
}

RTPSource::RTPSource(u_int32_t srcid)
	: next(0), np_(0), snp_(0), ehsr_(-1), LSR_(0), SRT_(0)
{
	bind("srcid_", (int*)&srcid_);
	srcid_ = srcid;
	
}

/**
 * Constructor
 * @param srcid the source id of the newly created RTPReceiver
 */
RTPReceiver::RTPReceiver(u_int32_t srcid)
	: next(0),eff_rtt_(0),rate_(0)
{
	srcid_ = srcid;
}
// Simulcast extensions
/**
 * Check if the receiver exists in the sender's list
 * @param srcid  the source id of this receiver
 * @return  the receiver if it exists,
 *  zero otherwise
 */
RTPReceiver* RTPSession::lookup_rcv(u_int32_t srcid)
{

        RTPReceiver *p;
        for (p = receivers_; p != 0; p = p->next)
        {
                if (p->srcid() == srcid)
                        return (p);
        }
        return (0);
}
/**
 *   enter a new receiver
 * @param s the new RTPReceiver
 */
void RTPSession::enter_rcv(RTPReceiver* s)
{
//	printf("Time:%f Sender:%d ...I add receiver %d \n", NOW,localsrc_->srcid(), s->srcid());
        s->next = receivers_;
        receivers_ = s;
	
}




void RTPSession::set_frametype(int fr_type)
{
	
	Tcl::instance().evalf("%s frame_type %d",name(),fr_type);
}

void RTPSession::send(int nbytes, int frame_number)
{
	Tcl::instance().evalf("%s send_nbytes %d %d",name(),nbytes,frame_number);
}

void RTPSession::set_quant(int quant)
{
	Tcl::instance().evalf("%s quant %d",name(),quant);
}

void RTPSession::set_join_flag(int a)
{
	join_flag = a;
}

void RTPSession::set_initial_rate(double value)
{
	smooth_rate=value/8; // convert from bps to Bps
	//printf("smooth_rate %f \n", smooth_rate);
}







