#
# Copyright (c) 2007 University of Patras Computer Engineering and Informatics Department Greece
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
# 	This product includes software developed by the MASH Research
# 	Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#


proc mvar args {
	upvar self _s
	uplevel $_s instvar $args
}

ASSP/Agent set uniq_srcid 0
ASSP/Agent proc alloc_srcid {} {
	set id [ASSP/Agent set uniq_srcid]
	ASSP/Agent set uniq_srcid [expr $id+1]
	return $id
}

ASSP/Agent instproc init {} {
	$self next
	mvar Stream1_ Stream2_ Stream3_  
	
	set Stream1_ [new Session/RTP] 
	set Stream2_ [new Session/RTP]
	set Stream3_ [new Session/RTP]
	
	
	set srcid_ [ASSP/Agent alloc_srcid]
	$self stream-id $srcid_
#	$self set streamid_  $streamid_
	
	$self 	  session $Stream1_ 
	$self 	  session $Stream2_
	$self 	  session $Stream3_
	$Stream1_ assp-agent $self
	$Stream2_ assp-agent $self
	$Stream3_ assp-agent $self
	$Stream1_ quality_level low
	$Stream2_ quality_level medium
	$Stream3_ quality_level high
	$self set stopped_ 1
}

ASSP/Agent instproc join-groups { g1 g2 g3 } {
#	set g [expr $g]
	mvar group1_ group2_ group3_ Stream1_ Stream2_  Stream3_
	$self set group1_ $g1
	$Stream1_ join-group $g1
#	incr g
#	incr g
#	incr g
	$self set group2_ $g2
	$Stream2_ join-group $g2

	$self set group3_ $g3
	$Stream3_ join-group $g3
#	incr g
#	incr g
#	incr g
#	puts "Server <-----------------------------------"
#	puts [$Stream1_ set srcid_]
#	puts [$self set group1_]
#	puts [$Stream2_ set srcid_]
#	puts [$self set group2_]
#	puts "Server ----------------------------------->"
}

ASSP/Agent instproc start {} {
	mvar group1_  
	if ![info exists group1_] {
		puts "error: can't transmit before joining group!"
		exit 1
	}
	mvar group2_
	if ![info exists group2_] {
		puts "error: can't transmit before joining group!"
		exit 1
	}
	mvar group3_
	if ![info exists group3_] {
		puts "error: can't transmit before joining group!"
		exit 1
	}
	mvar Stream1_  Stream2_  Stream3_
	$Stream1_ start 
	$Stream2_ start
	$Stream3_ start  
	$self start-timer 
}


ASSP/Agent instproc stop {} {
	$self instvar Stream1_ Stream2_  Stream3_
	$Stream1_ stop
	$Stream2_ stop
	$Stream3_ stop
	$self set stopped_ 1
}

ASSP/Agent instproc attach-node { node } {
	mvar Stream1_ Stream2_ Stream3_ 
	$Stream1_ attach-node $node 
	$Stream2_ attach-node $node 
	$Stream3_ attach-node $node 
	$self set node_ $node
}

ASSP/Agent instproc detach-node { node } {
	mvar Stream1_ Stream2_ Stream3_
	$Stream1_ detach-node $node 
	$Stream2_ detach-node $node
	$Stream3_ detach-node $node  
	$self unset node_
}



ASSP/Agent instproc set_filename_low {filename} {
	mvar Stream1_ 
	$Stream1_ set_filename $filename
}

ASSP/Agent instproc set_filename_med {filename} {
	mvar Stream2_ 
	$Stream2_ set_filename $filename
}

ASSP/Agent instproc set_filename_high {filename} {
	mvar Stream3_ 
	$Stream3_ set_filename $filename
}

ASSP/Agent instproc set_sender {value} {
	mvar Stream1_ Stream2_ Stream3_ 
	$Stream1_ set_sender $value
	$Stream2_ set_sender $value
	$Stream3_ set_sender $value
}

ASSP/Agent instproc enable-control {l} {
	mvar Stream1_  Stream2_ Stream3_
	$Stream1_ enable-control $l 
	$Stream2_ enable-control $l 
	$Stream3_ enable-control $l 
}








ASSP/Agent instproc transmit-low {l} {
	mvar Stream1_ 
	$Stream1_ transmit $l 
}
	
ASSP/Agent instproc transmit-medium {l} {
	mvar Stream2_ 
	$Stream2_ transmit $l 
}
	
ASSP/Agent instproc transmit-high {l} {
	mvar Stream3_ 
	$Stream3_ transmit $l 
}


ASSP/Agent  instproc session_bw-low {l} {
	mvar Stream1_ 
	$Stream1_ session_bw $l 
}
	
ASSP/Agent instproc session_bw-medium {l} {
	mvar Stream2_ 
	$Stream2_ session_bw $l 
}

ASSP/Agent  instproc session_bw-high {l} {
	mvar Stream3_ 
	$Stream3_ session_bw $l 
}



ASSP/Agent  set srcid_  -1


