#
# Copyright (c) 2007 University of Patras Computer Engineering and Informatics Department Greece
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
# 	This product includes software developed by the MASH Research
# 	Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#


proc mvar args {
	upvar self _s
	uplevel $_s instvar $args
}

ASSP/Sink set uniq_srcid 1
ASSP/Sink proc alloc_srcid {} {
	set id [ASSP/Sink set uniq_srcid]
	ASSP/Sink set uniq_srcid [expr $id+1]
	return $id
}



ASSP/Sink instproc init {} {
	$self next
	mvar Stream1_ Stream2_ Stream3_  
	
	set Stream1_ [new Session/RTP] 
	set Stream2_ [new Session/RTP]
	set Stream3_ [new Session/RTP]
	
	
	set srcid_ [ASSP/Sink alloc_srcid]
	$self stream-id $srcid_
#	$self set streamid_  $streamid_

	$self 	  session $Stream1_ 
	$self 	  session $Stream2_
	$self 	  session $Stream3_
	$Stream1_ assp-sink $self
	$Stream2_ assp-sink $self
	$Stream3_ assp-sink $self
	$self set stopped_ 1
}

ASSP/Sink instproc assign-groups { g1 g2 g3 } {
	mvar group1_ group2_ group3_ 
#	set g1 [expr $g1]
#	set g2 [expr $g2]
#	set g3 [expr $g3]
	$self set group1_ $g1
	$self set group2_ $g2
	$self set group3_ $g3
#	puts "Client <-----------------------------------"
#	puts [$self set group1_]
#	puts [$self set group2_]
#	puts [$self set group3_]
#	puts "Client ----------------------------------->"
}

ASSP/Sink instproc join-group { g } {
	mvar group1_ group2_ group3_ Stream1_ Stream2_  Stream3_
#	set g [expr $g]
	if {$group1_ == $g} {
		$self set group1_ $g
		$Stream1_ join-group $g
	}
	mvar group2_ Stream2_
	if {$group2_ == $g} {
		$self set group2_ $g
		$Stream2_ join-group $g
	}
	mvar group3_ Stream3_
	if {$group3_ == $g} {
		$self set group3_ $g
		$Stream3_ join-group $g
	}
}

ASSP/Sink instproc start-session { g } {
	mvar group1_  Stream1_ 
	if ![info exists group1_] {
		puts "error: can't transmit before joining group!"
		exit 1
	} 
	if {$group1_ == $g} {
		$Stream1_ start 
	}
	mvar group2_ Stream2_
	if ![info exists group2_] {
		puts "error: can't transmit before joining group!"
		exit 1
	}
	if {$group2_ == $g} {
		$Stream2_ start 
	}
	mvar group3_ Stream3_
	if ![info exists group3_] {
		puts "error: can't transmit before joining group!"
		exit 1
	}
	if {$group3_ == $g} {
		$Stream3_ start 
	}
}

ASSP/Sink instproc stop-group { g } {
	mvar group1_  Stream1_ 
	if {$group1_ == $g} {
		$Stream1_ stop 
	}
	mvar group2_ Stream2_
	if {$group2_ == $g} {
		$Stream2_ stop 
	}
	mvar group3_ Stream3_
	if {$group3_ == $g} {
		$Stream3_ stop 
	}
}

ASSP/Sink instproc bye-group { g } {
	mvar group1_  Stream1_ 
	if {$group1_ == $g} {
		$Stream1_ bye 
	}
	mvar group2_ Stream2_
	if {$group2_ == $g} {
		$Stream2_ bye 
	}
	mvar group3_ Stream3_
	if {$group3_ == $g} {
		$Stream3_ bye 
	}
}


ASSP/Sink instproc attach-node { node } {
	mvar Stream1_ Stream2_ Stream3_ 
	$Stream1_ attach-node $node 
	$Stream2_ attach-node $node 
	$Stream3_ attach-node $node 
	$self set node_ $node
}

ASSP/Sink instproc detach-node { node } {
	mvar Stream1_ Stream2_ Stream3_
	$Stream1_ detach-node $node 
	$Stream2_ detach-node $node
	$Stream3_ detach-node $node  
	$self unset node_
}



ASSP/Sink instproc leave-group { g } {
	mvar group1_  Stream1_ 
	if {$group1_ == $g} {
		$Stream1_ leave-group 
#		$self unset group1_
	}
	mvar group2_ Stream2_
	if {$group2_ == $g} {
		$Stream2_ leave-group 
#		$self unset group2_
	}
	mvar group3_ Stream3_
	if {$group3_ == $g} {
		$Stream3_ leave-group 
#		$self unset group3_
	}
}


ASSP/Sink instproc session_bw-low {l} {
	mvar Stream1_ 
	$Stream1_ session_bw $l 
}
	
ASSP/Sink instproc session_bw-medium {l} {
	mvar Stream2_ 
	$Stream2_ session_bw $l 
}

ASSP/Sink instproc session_bw-high {l} {
	mvar Stream3_ 
	$Stream3_ session_bw $l 
}

ASSP/Sink instproc go-up { l } {
	mvar group1_ group2_ group3_ Stream1_ Stream2_  Stream3_ level_
	if { $l == 0} {
		
		$self bye-group $group1_ 
		$self leave-group $group1_
		$self set level_ 1 
		$self join-group $group2_
		$self start-session $group2_ 
		
		
	}
	if { $l == 1 } {
		
		$self bye-group $group2_ 
		$self leave-group $group2_
		$self set level_ 2 
		$self join-group $group3_
		$self start-session $group3_ 
		
		
	}
}

ASSP/Sink instproc go-down { l } {
	mvar group1_ group2_ group3_ Stream1_ Stream2_  Stream3_ level_
	if { $l == 2 } {
		
		$self bye-group $group3_ 
		$self leave-group $group3_
		$self set level_ 1
		$self join-group $group2_
		$self start-session $group2_
		
		
	}
	if { $l == 1 } {
		
		$self bye-group $group2_ 
		$self leave-group $group2_
		$self set level_ 0 
		$self join-group $group1_
		$self start-session $group1_ 
		
		
	}
}



ASSP/Sink  set srcid_  -1


