










#===================================
# Simulation with competting UDP traffic
# Author: G. Kioumourtzis
#===================================





#Create a ns simulator
set ns [new Simulator -multicast on]


$ns color 1 blue
$ns color 2 red
$ns color 3 yellow
$ns color 30 purple
$ns color 31 bisque
$ns color 32 green
#Open the NS trace file
#set tracefile [open out.tr w]
#$ns trace-all $tracefile

set t1 [open assp_s1.txt w]
set t3 [open assp_s3.txt w]
set t5 [open assp_s5.txt w]
set f0 [open udp_trace.txt w]


#Open the NAM trace file
set namfile [open out.nam w]
$ns namtrace-all $namfile

#===================================
#     Simulation parameters setup
#===================================
set simtime  80
set opt(low)	500000   ;#in bps
set opt(medium)	1000000	  ;#in bps
set opt(high)   2000000   ;#in bps
set opt(report_interval) 1000ms

set opt(low_threshold) 250000
set opt(medium_threshold) 500000
set opt(high_threshold) 1000000

set max_fragmented_size   972
set frames_per_second 25   ;# Evalvid-RA, All video is treated with equal fps in current release
set q_variants 30 ;# Evalvid-RA, number of quantiser scale range
#set vbr_rate 2.0e6 ;#500000   ;# Evalvid-RA video: select base rate on VBR sources (includes IP overhead)
# router buffer



#===================================
#        Nodes Definition        
#===================================
#Create 11 nodes

for {set i 0} {$i < 17} {incr i} {
    set n$i [$ns node]
    
}


# Add some shapes
$n1 shape "box"
$n2 shape "box"
$n3 shape "box"
$n4 shape "box"
$n0 label "Multicast-Server"
$n5 label "Multicast-Receiver"
$n6 label "Multicast-Receiver"
$n7 label "Multicast-Receiver"
$n8 label "Multicast-Receiver"
$n9 label "Multicast-Receiver"
$n10 label "Multicast-Receiver"



#===================================
#       Set-up Links
#===================================
# Link capacities
set access_cap 10Mb       ;# access link capacities
# Link propagation delays
set access_del     5ms
set bottleneck_del 30ms
# This is the size of the queue buffer:
set queueTime 0.4     
set bottleneck_cap0 4Mb
set bottleneck_cap1 2Mb 
set bottleneck_cap2 1Mb
set bottleneck_del0 10ms
set bottleneck_del1 20ms
set bottleneck_del2 40ms

set packetSize	[expr $max_fragmented_size + 40]
#set queueSize1 [expr ($bottleneck_cap0*$queueTime)/(8*$packetSize)] 
#set queueSize2 [expr ($bottleneck_cap1*$queueTime)/(8*$packetSize)] 
#set queueSize3 [expr ($bottleneck_cap2*$queueTime)/(8*$packetSize)] 

$ns duplex-link $n0 $n1  100Mb  $access_del DropTail 
$ns duplex-link $n1 $n2  $bottleneck_cap0  $bottleneck_del DropTail ;#$queueSize1
$ns duplex-link $n2 $n3  $bottleneck_cap1  $bottleneck_del DropTail  ;#$queueSize2
$ns duplex-link $n3 $n4  $bottleneck_cap2  $bottleneck_del DropTail ;#$queueSize3


# set up ASSP Sink links
$ns duplex-link $n2 $n5  $access_cap  $access_del DropTail
$ns duplex-link $n2 $n6  $access_cap  $access_del DropTail
$ns duplex-link $n3 $n7  $access_cap  $access_del DropTail
$ns duplex-link $n3 $n8  $access_cap  $access_del DropTail
$ns duplex-link $n4 $n9  $access_cap  $access_del DropTail
$ns duplex-link $n4 $n10  $access_cap  $access_del DropTail

# set up TCP links
$ns duplex-link $n1 $n11  $access_cap  $access_del DropTail
$ns duplex-link $n2 $n14  $access_cap  $access_del DropTail
$ns duplex-link $n2 $n12  $access_cap  $access_del DropTail
$ns duplex-link $n3 $n13  $access_cap  $access_del DropTail
$ns duplex-link $n3 $n15  $access_cap  $access_del DropTail
$ns duplex-link $n4 $n16  $access_cap  $access_del DropTail


#######################################################################
# Set-up Simulcast sessions                                #
#######################################################################
# configure multicast protocol;
#set mproto CtrMcast
set mproto DM
# all nodes will contain multicast protocol agents;
set mrthandle [$ns mrtproto $mproto]
# allocate a multicast address;
set group1 [Node allocaddr]
set group2 [Node allocaddr]
set group3 [Node allocaddr]


set s0 [new ASSP/Agent ]
#$s0 report-interval $opt(report_interval)
$s0 attach-node $n0
$s0 set_filename_low  sd_be_0
$s0 set_filename_med  sd_be_1
$s0 set_filename_high sd_be_2
$s0 set_sender true
$s0 set timeout 5
$s0 set low_tx [expr $opt(low)/8]  
$s0 set medium_tx [expr $opt(medium)/8]  
$s0 set high_tx [expr $opt(high)/8]
# enable congestion control
$s0 enable-control 1

set s1 [new ASSP/Sink ]
$s1 attach-node $n5
$s1 set_filename rd_be_1
$s1 set level_ 0
$s1 set_initial_rate  $opt(high)
$s1 set low_threshold [expr $opt(low_threshold)/8] 
$s1 set medium_threshold expr $opt(medium_threshold)/8] 
$s1 set high_threshold [expr $opt(high_threshold)/8]

set s2 [new ASSP/Sink ]
$s2 attach-node $n6
$s2 set_filename rd_be_2
$s2 set level_ 0
$s2 set_initial_rate $opt(high)  
$s2 set low_threshold [expr $opt(low_threshold)/8]  
$s2 set medium_threshold [expr $opt(medium_threshold)/8] 
$s2 set high_threshold [expr $opt(high_threshold)/8]

set s3 [new ASSP/Sink ]
$s3 attach-node $n7
$s3 set_filename rd_be_3
$s3 set level_ 0
$s3 set_initial_rate $opt(medium) 
$s3 set low_threshold [expr $opt(low_threshold)/8]   
$s3 set medium_threshold expr $opt(medium_threshold)/8] 
$s3 set high_threshold [expr $opt(high_threshold)/8]

set s4 [new ASSP/Sink ]
$s4 attach-node $n8
$s4 set_filename rd_be_4
$s4 set level_ 0
$s4 set_initial_rate $opt(medium) 
$s4 set low_threshold [expr $opt(low_threshold)/8]  
$s4 set medium_threshold expr $opt(medium_threshold)/8] 
$s4 set high_threshold [expr $opt(high_threshold)/8]

set s5 [new ASSP/Sink ]
$s5 attach-node $n9
$s5 set_filename rd_be_5
$s5 set level_ 0
$s5 set_initial_rate $opt(low)
$s5 set low_threshold [expr $opt(low_threshold)/8]  
$s5 set medium_threshold expr $opt(medium_threshold)/8] 
$s5 set high_threshold [expr $opt(high_threshold)/8]

set s6 [new ASSP/Sink ]
$s6 attach-node $n10
$s6 set_filename rd_be_6
$s6 set level_ 0
$s6 set_initial_rate $opt(low)
$s6 set low_threshold [expr $opt(low_threshold)/8]  
$s6 set medium_threshold expr $opt(medium_threshold)/8] 
$s6 set high_threshold [expr $opt(high_threshold)/8]


#===================================
#       Set-up background traffic
#===================================
#Setup a UDP connection
#Agent/UDP set class_ 1
set udp0 [new Agent/UDP]
$ns attach-agent $n11 $udp0
set sink0 [new Agent/LossMonitor]
$ns attach-agent $n14 $sink0
$ns connect $udp0 $sink0
#$udp0 set packetSize_ 1064
#Setup a FTP Application over TCP connection
set ftp0 [new Application/Traffic/CBR]
#set ftp0 [new Application/FTP]
$ftp0 attach-agent $udp0
$ftp0 set rate_  0.5Mb/s
$ftp0 set packetSize_ 984
$ns at 10.0  "$ftp0 start"
$ns at 20.0 "$ftp0 stop"
$ns at 30.0 "$ftp0 start"
$ns at 40.0 "$ftp0 stop"
$ns at 50.0 "$ftp0 start"
$ns at 60.0 "$ftp0 stop"

#Setup a UDP connection
#Agent/UDP set class_ 1
set udp1 [new Agent/UDP]
$ns attach-agent $n12 $udp1
set sink1 [new Agent/LossMonitor]
$ns attach-agent $n15 $sink1
$ns connect $udp1 $sink1
#$udp0 set packetSize_ 1064
#Setup a FTP Application over TCP connection
set ftp1 [new Application/Traffic/CBR]
#set ftp0 [new Application/FTP]
$ftp1 attach-agent $udp1
$ftp1 set rate_  0.5Mb/s
$ftp1 set packetSize_ 984
$ns at 5.0  "$ftp1 start"
$ns at 10.0 "$ftp1 stop"
$ns at 15.0 "$ftp1 start"
$ns at 20.0 "$ftp1 stop"
$ns at 25.0 "$ftp1 start"
$ns at 30.0 "$ftp1 stop"
$ns at 35.0 "$ftp1 start"
$ns at 40.0 "$ftp1 stop"
$ns at 35.0 "$ftp1 start"
$ns at 40.0 "$ftp1 stop"
$ns at 45.0 "$ftp1 start"
$ns at 50.0 "$ftp1 stop"
$ns at 55.0 "$ftp1 start"
$ns at 60.0 "$ftp1 stop"
$ns at 65.0 "$ftp1 start"
$ns at 70.0 "$ftp1 stop"

#Setup a UDP connection
#Agent/UDP set class_ 1
set udp2 [new Agent/UDP]
$ns attach-agent $n13 $udp2
set sink2 [new Agent/LossMonitor]
$ns attach-agent $n16 $sink2
$ns connect $udp2 $sink2
#$udp0 set packetSize_ 1064
#Setup a FTP Application over TCP connection
set ftp2 [new Application/Traffic/CBR]
#set ftp0 [new Application/FTP]
$ftp2 attach-agent $udp2
$ftp2 set rate_  0.5Mb/s
$ftp2 set packetSize_ 984
$ns at 0.0  "$ftp2 start"
$ns at 5.0 "$ftp2 stop"
$ns at 1.0  "$ftp2 start"
$ns at 15.0 "$ftp2 stop"
$ns at 2.0  "$ftp2 start"
$ns at 25.0 "$ftp2 stop"
$ns at 30.0  "$ftp2 start"
$ns at 35.0 "$ftp2 stop"
$ns at 40.0  "$ftp2 start"
$ns at 45.0 "$ftp2 stop"
$ns at 50.0  "$ftp2 start"
$ns at 55.0 "$ftp2 stop"
$ns at 60.0  "$ftp2 start"
$ns at 65.0 "$ftp2 stop"
$ns at 70.0  "$ftp2 start"
$ns at 75.0 "$ftp2 stop"




#################################################
# Create Evalvid-RA traffic sources             #
#################################################
##
## VBR0 as VBR rate adaptiv traffic
##
# The followong file is used as ns-2 adapted (by Chih-Heng, Ke from Taiwan) version of 
# the Evalvid (J. Klaue) generated file above
# Sender trace file from mp4.exe containing frame type and size
puts "Evalvid-RA: Start making GOP and Frame trace files for $q_variants Rate variants"
for {set i 1} {$i <= $q_variants} {incr i} {
    set original_file_name($i) st_highway_cif.yuv_Q[expr $i + 1].txt 
    set original_file_id($i) [open $original_file_name($i) r]
}
set trace_file_name video2.dat
set trace_file_id [open $trace_file_name w]
set trace_file [new vbrTracefile2]
$trace_file filename $trace_file_name
set frame_count 0
set last_time 0

# AL: toggle between multiple input files!
#set original_file_id $original_file_id(1)
#set source_select 1


set frame_size_file frame_size.dat
set frame_size_file_id [open $frame_size_file w]
for {set i 1} {$i <= $q_variants} {incr i} {
    set frame_size($i) 0
}
set gop_size_file gop_size.dat
set gop_size_file_id [open $gop_size_file w]
for {set i 1} {$i <= $q_variants} {incr i} {
    set gop_size($i) 0
}
set gop_numb 0
# Convert ASCII sender file on frame size granularity to ns-2 adapted internal format
while {[eof $original_file_id(1)] == 0} {
    for {set i 1} {$i <= $q_variants} {incr i} {
	gets $original_file_id($i) current_line($i)
    }
    
    scan $current_line(1) "%d%s%d%s%s%s%d%s" no_ frametype_ length_ tmp1_ tmp2_ tmp3_ tmp4_ tmp5_

#    puts "$no_ $frametype_ $length_ $tmp1_ $tmp2_ $tmp3_ $tmp4_ $tmp5_"
        
    # 30 frames/sec. if one want to generate 25 frames/sec, one can use set time [expr 1000*1000/25]
    set tempStr "%.0f"
    set time [expr 1000 * 1000/$frames_per_second] 
#    puts $f5 [format $tempStr $time]
    set time [format $tempStr $time]
    

    if { $frametype_ == "I" } {
  	set type_v 1
	set time 0
    }	

    if { $frametype_ == "P" } {
  	set type_v 2
    }	

    if { $frametype_ == "B" } {
  	set type_v 3
    }	
    
    # Write to GOP size file after each H-frame found:
    if { $frametype_ == "H" } {
	set puts_string "$gop_numb"
	for {set i 1} {$i <= $q_variants} {incr i} {
	    set puts_string "$puts_string $gop_size($i)" 
	}
	puts $gop_size_file_id $puts_string
	set gop_numb [expr $gop_numb + 1]
  	set type_v 0 ;# Must have different type than I-frame so that the LB(r,b) algorithm finds it!
    }	
# Write to frame_size.dat:
    set puts_string "$no_"
    for {set i 1} {$i <= $q_variants} {incr i} {
	set puts_string "$puts_string $gop_size($i)" 
    }
    puts $frame_size_file_id $puts_string

    for {set i 1} {$i <= $q_variants} {incr i} {
	scan $current_line($i) "%d%s%d%s%s%s%d%s" no_ frametype_ length($i) tmp1_ tmp2_ tmp3_ tmp4_ tmp5_
	set gop_size($i) [expr $gop_size($i) + $length($i) ]
    }

# Write to video2.dat:
    set puts_string "$time $length_ $type_v $max_fragmented_size"
    for {set i 2} {$i <= $q_variants} {incr i} {
	set puts_string "$puts_string $length($i)"
    }
    puts  $trace_file_id $puts_string
    incr frame_count
 
}
puts "Evalvid-RA: #of frames written to GOP and Frame trace files: $frame_count"
#close $original_file_id
close $trace_file_id  ;# Note that this new trace file is closed for writing and 
# opened below for reading through being a new Tracefile in eraTraceFile2::setup()



set vbr0 [new Application/Traffic/eraVbrTrace]
	$vbr0 set running_ 0 
	$vbr0 set r_ [expr $opt(medium) ]   ;# Set the rate instead of packet interval, 
	#$vbr($i) set r_ [expr $vbr_rate + 0.0]   ;# Set the rate instead of packet interval, 
	
	#$vbr($i) set r_ [expr $vbr_rate + $i*0.5e3]   ;# Set the rate instead of packet interval, 
	;# eraVbrTrace will calculate the interval.
	$vbr0 attach-tracefile $trace_file
	#    $vbr($i) set b_ [expr $vbr_rate * 1.5] 
	$vbr0 set b_ 1.0
	$vbr0 set q_ 2
	#$vbr0 set q_ 2
	$vbr0 set GoP_ 12
	$vbr0 set fps_ $frames_per_second
	$vbr0 set q_level_ 0
	$vbr0 assp-agent $s0
	$s0 attach-vbr $vbr0 

set vbr1 [new Application/Traffic/eraVbrTrace]
	$vbr1 set running_ 0 
	$vbr1 set r_ [expr $opt(medium) ]   ;# Set the rate instead of packet interval, 
	#$vbr($i) set r_ [expr $vbr_rate + 0.0]   ;# Set the rate instead of packet interval, 
	
	#$vbr($i) set r_ [expr $vbr_rate + $i*0.5e3]   ;# Set the rate instead of packet interval, 
	;# eraVbrTrace will calculate the interval.
	$vbr1 attach-tracefile $trace_file
	#    $vbr($i) set b_ [expr $vbr_rate * 1.5] 
	$vbr1 set b_ 1.0
	$vbr1 set q_ 2
	#$vbr0 set q_ 2
	$vbr1 set GoP_ 12
	$vbr1 set fps_ $frames_per_second
	$vbr1 set q_level_ 1
	$vbr1 assp-agent $s0
	$s0 attach-vbr $vbr1

set vbr2 [new Application/Traffic/eraVbrTrace]
	$vbr2 set running_ 0
	$vbr2 set r_ [expr $opt(high) ]   ;# Set the rate instead of packet interval, 
	#$vbr($i) set r_ [expr $vbr_rate + 0.0]   ;# Set the rate instead of packet interval, 
	
	#$vbr($i) set r_ [expr $vbr_rate + $i*0.5e3]   ;# Set the rate instead of packet interval, 
	;# eraVbrTrace will calculate the interval.
	$vbr2 attach-tracefile $trace_file
	#    $vbr($i) set b_ [expr $vbr_rate * 1.5] 
	$vbr2 set b_ 1.0
	$vbr2 set q_ 2
#	$vbr0 set q_ 2
	$vbr2 set GoP_ 12
	$vbr2 set fps_ $frames_per_second
	$vbr2 set q_level_ 2
	$vbr2 assp-agent $s0
	$s0 attach-vbr $vbr2 
 


$ns at 0.0 "$s0 join-groups $group1 $group2 $group3"
$ns at 0.0 "$s0 start"
$ns at 0.0 "$s0 start-timer"

$ns at 0.0 "$s1 assign-groups $group1 $group2 $group3"
$ns at 0.0 "$s1 join-group $group1"
$ns at 0.0 "$s1 start-session $group1"

$ns at 0.0 "$s2 assign-groups $group1 $group2 $group3"
$ns at 0.0 "$s2 join-group $group1"
$ns at 0.0 "$s2 start-session $group1"

$ns at 0.0 "$s3 assign-groups $group1 $group2 $group3"
$ns at 0.0 "$s3 join-group $group1"
$ns at 0.0 "$s3 start-session $group1"

$ns at 0.0 "$s4 assign-groups $group1 $group2 $group3"
$ns at 0.0 "$s4 join-group $group1"
$ns at 0.0 "$s4 start-session $group1"

$ns at 0.0 "$s5 assign-groups $group1 $group2 $group3"
$ns at 0.0 "$s5 join-group $group1"
$ns at 0.0 "$s5 start-session $group1"

$ns at 0.0 "$s6 assign-groups $group1 $group2 $group3"
$ns at 0.0 "$s6 join-group $group1"
$ns at 0.0 "$s6 start-session $group1"

$ns at 0.0       "$vbr0 start"
$ns at $simtime  "$vbr0 stop"

$ns at 0.0       "$vbr1 start"
$ns at $simtime  "$vbr1 stop"

$ns at 0.0       "$vbr2 start"
$ns at $simtime  "$vbr2 stop"




#===================================
#        Record functions
#===================================

$ns at 0.0 "record-s1"
$ns at 0.0 "record-s3"
$ns at 0.0 "record-s5"
$ns at 0.0 "record-sink0"

proc record-s1 {} {
        global s1 t1
	#Get an instance of the simulator
	set ns [Simulator instance]
	#Set the time after which the procedure should be called again
        set time 1.5
	#How many bytes have been received by the traffic?
	set bw0 [$s1 set rcv_bytes]
        #Get the current time
        set now [$ns now]
	puts "$now"
#	puts "$now"
	#Calculate the bandwidth (in MBit/s) and write it to the files
	puts $t1 "$now [expr $bw0/$time*8]"
        #Reset the bytes_ values on the traffic sinks
        $s1 set rcv_bytes 0
       	#Re-schedule the procedure
        $ns at [expr $now+$time] "record-s1"
}

proc record-s3 {} {
        global s3 t3
	#Get an instance of the simulator
	set ns [Simulator instance]
	#Set the time after which the procedure should be called again
        set time 1.5
	#How many bytes have been received by the traffic?
	set bw0 [$s3 set rcv_bytes]
        #Get the current time
        set now [$ns now]
	#puts "$now"
#	puts "$now"
	#Calculate the bandwidth (in MBit/s) and write it to the files
	puts $t3 "$now [expr $bw0/$time*8]"
        #Reset the bytes_ values on the traffic sinks
        $s3 set rcv_bytes 0
       	#Re-schedule the procedure
        $ns at [expr $now+$time] "record-s3"
}

proc record-s5 {} {
        global s5 t5
	#Get an instance of the simulator
	set ns [Simulator instance]
	#Set the time after which the procedure should be called again
        set time 1.5
	#How many bytes have been received by the traffic?
	set bw0 [$s5 set rcv_bytes]
        #Get the current time
        set now [$ns now]
	#puts "$now"
#	puts "$now"
	#Calculate the bandwidth (in MBit/s) and write it to the files
	puts $t5 "$now [expr $bw0/$time*8]"
        #Reset the bytes_ values on the traffic sinks
        $s5 set rcv_bytes 0
       	#Re-schedule the procedure
        $ns at [expr $now+$time] "record-s5"
}

# record the UDP throughput
proc record-sink0 {} {
        global sink0 f0
	#Get an instance of the simulator
	set ns [Simulator instance]
	#Set the time after which the procedure should be called again
        set time 0.5
	#How many bytes have been received by the traffic?
	set bw0 [$sink0 set bytes_]
        #Get the current time
        set now [$ns now]
	#Calculate the bandwidth (in MBit/s) and write it to the files
	puts $f0 "$now [expr $bw0/$time*8]"
        #Reset the bytes_ values on the traffic sinks
        $sink0 set bytes_ 0
       	#Re-schedule the procedure
        $ns at [expr $now+$time] "record-sink0"
}



#===================================
#        Termination        
#===================================
#Define a 'finish' procedure
proc finish {} {
     global ns  namfile t1 t3 t5 f0
     $ns flush-trace
#     close $tracefile
     close $namfile
     close $t1
     close $t3
     close $t5
     close $f0
    
	 exec xgraph   assp_s1.txt assp_s3.txt assp_s5.txt udp_trace.txt -geometry 800x400  &
     #exec nam out.nam &
     exit 0
}



$ns at [expr $simtime+10] "finish"



$ns run
