#===================================
#      Simulation to test the 
# intra-fairness of ASSP
# Author:G. Kioumourtzis
#===================================
#Create a ns simulator
set ns [new Simulator -multicast on]


$ns color 1 blue
$ns color 2 red
$ns color 3 yellow
$ns color 30 purple
$ns color 31 bisque
$ns color 32 green
#Open the NS trace file
set tracefile [open out.tr w]
$ns trace-all $tracefile

set f1 [open througput_s1.txt w]
set f3 [open througput_s3.txt w]
set f5 [open througput_s5.txt w]
set f6 [open througput_r1.txt w]
set f7 [open througput_r3.txt w]
set f8 [open througput_r5.txt w]

#Open the NAM trace file
set namfile [open out.nam w]
$ns namtrace-all $namfile

#===================================
#     Create trace files
#===================================



#===================================
#     Simulation parameters setup
#===================================
set simtime  80
set opt(low)	500000   ;#in bps
set opt(medium)	1000000	  ;#in bps
set opt(high)   2000000   ;#in bps
set opt(report_interval) 1000ms

set opt(low_threshold) 250000
set opt(medium_threshold) 500000
set opt(high_threshold) 1000000
set max_fragmented_size   972
set frames_per_second 25   ;# Evalvid-RA, All video is treated with equal fps in current release
set q_variants 30 ;# Evalvid-RA, number of quantiser scale range
#set vbr_rate 2.0e6 ;#500000   ;# Evalvid-RA video: select base rate on VBR sources (includes IP overhead)
# router buffer



#===================================
#        Nodes Definition        
#===================================
#Create 11 nodes

for {set i 0} {$i < 18} {incr i} {
    set n$i [$ns node]
    
}


# Add some shapes
$n1 shape "box"
$n2 shape "box"
$n3 shape "box"
$n4 shape "box"
$n0 label "Multicast-Server"
$n5 label "Multicast-Receiver"
$n6 label "Multicast-Receiver"
$n7 label "Multicast-Receiver"
$n8 label "Multicast-Receiver"
$n9 label "Multicast-Receiver"
$n10 label "Multicast-Receiver"



#===================================
#       Set-up Links
#===================================
# Link capacities
set access_cap 10Mb       ;# access link capacities
# Link propagation delays
set access_del     5ms
set bottleneck_del1 10ms
set bottleneck_del2 20ms
set bottleneck_del3 40ms
# This is the size of the queue buffer:
set queueTime 0.4     
set bottleneck_cap1 4Mb
set bottleneck_cap2 2Mb   
set bottleneck_cap3 1Mb 
set packetSize	[expr $max_fragmented_size + 40]
set queueSize1 [expr (4000000*$queueTime)/(8*$packetSize)] 
set queueSize2 [expr (2000000*$queueTime)/(8*$packetSize)] 
set queueSize3 [expr (1000000*$queueTime)/(8*$packetSize)] 

puts "$queueSize1"
puts "$queueSize2"
puts "$queueSize3"

$ns duplex-link $n0 $n1  100Mb  $access_del DropTail 
$ns duplex-link $n5 $n1  100Mb  $access_del DropTail 


$ns duplex-link $n2 $n6  $access_cap  $access_del DropTail
$ns duplex-link $n2 $n7  $access_cap  $access_del DropTail
$ns duplex-link $n2 $n14  $access_cap  $access_del DropTail
$ns duplex-link $n2 $n15  $access_cap  $access_del DropTail

$ns duplex-link $n3 $n8  $access_cap  $access_del DropTail
$ns duplex-link $n3 $n9  $access_cap  $access_del DropTail
$ns duplex-link $n3 $n16  $access_cap  $access_del DropTail
$ns duplex-link $n3 $n17  $access_cap  $access_del DropTail

$ns duplex-link $n4 $n10  $access_cap  $access_del DropTail
$ns duplex-link $n4 $n11  $access_cap  $access_del DropTail
$ns duplex-link $n4 $n12  $access_cap  $access_del DropTail
$ns duplex-link $n4 $n13  $access_cap  $access_del DropTail

$ns duplex-link $n1 $n2  $bottleneck_cap1  $bottleneck_del1 DropTail $queueSize1
$ns duplex-link $n2 $n3  $bottleneck_cap2  $bottleneck_del2 DropTail $queueSize2
$ns duplex-link $n3 $n4  $bottleneck_cap3  $bottleneck_del3 DropTail $queueSize3



#$ns queue-limit $n1 $n2 

#######################################################################
# Set-up Simulcast sessions                                #
#######################################################################
# configure multicast protocol;
#set mproto CtrMcast
set mproto DM
# all nodes will contain multicast protocol agents;
set mrthandle [$ns mrtproto $mproto]
# allocate a multicast address;
set group1 [Node allocaddr]
set group2 [Node allocaddr]
set group3 [Node allocaddr]

set group4 [Node allocaddr]
set group5 [Node allocaddr]
set group6 [Node allocaddr]


set s0 [new ASSP/Agent ]
#$s0 report-interval $opt(report_interval)
$s0 attach-node $n0
$s0 set_filename_low  sd_be_0
$s0 set_filename_med  sd_be_1
$s0 set_filename_high sd_be_2
$s0 set_sender true
$s0 set timeout 5
$s0 set low_tx [expr $opt(low)/8]  
$s0 set medium_tx [expr $opt(medium)/8]  
$s0 set high_tx [expr $opt(high)/8]
# enable congestion control
$s0 enable-control 1

set s1 [new ASSP/Sink ]
$s1 attach-node $n6
$s1 set_filename rd_be_1
$s1 set level_ 0
$s1 set_initial_rate  $opt(high)
$s1 set low_threshold [expr $opt(low_threshold)/8] 
$s1 set medium_threshold expr $opt(medium_threshold)/8] 
$s1 set high_threshold [expr $opt(high_threshold)/8]

set s2 [new ASSP/Sink ]
$s2 attach-node $n7
$s2 set_filename rd_be_2
$s2 set level_ 0
$s2 set_initial_rate $opt(high)  
$s2 set low_threshold [expr $opt(low_threshold)/8]  
$s2 set medium_threshold [expr $opt(medium_threshold)/8] 
$s2 set high_threshold [expr $opt(high_threshold)/8]

set s3 [new ASSP/Sink ]
$s3 attach-node $n8
$s3 set_filename rd_be_3
$s3 set level_ 0
$s3 set_initial_rate $opt(medium) 
$s3 set low_threshold [expr $opt(low_threshold)/8]   
$s3 set medium_threshold expr $opt(medium_threshold)/8] 
$s3 set high_threshold [expr $opt(high_threshold)/8]

set s4 [new ASSP/Sink ]
$s4 attach-node $n9
$s4 set_filename rd_be_4
$s4 set level_ 0
$s4 set_initial_rate $opt(medium) 
$s4 set low_threshold [expr $opt(low_threshold)/8]  
$s4 set medium_threshold expr $opt(medium_threshold)/8] 
$s4 set high_threshold [expr $opt(high_threshold)/8]

set s5 [new ASSP/Sink ]
$s5 attach-node $n10
$s5 set_filename rd_be_5
$s5 set level_ 0
$s5 set_initial_rate $opt(low)
$s5 set low_threshold [expr $opt(low_threshold)/8]  
$s5 set medium_threshold expr $opt(medium_threshold)/8] 
$s5 set high_threshold [expr $opt(high_threshold)/8]

set s6 [new ASSP/Sink ]
$s6 attach-node $n11
$s6 set_filename rd_be_6
$s6 set level_ 0
$s6 set_initial_rate $opt(low)
$s6 set low_threshold [expr $opt(low_threshold)/8]  
$s6 set medium_threshold expr $opt(medium_threshold)/8] 
$s6 set high_threshold [expr $opt(high_threshold)/8]

#################################################
# Second ASSP Agent                             #
#################################################
set r0 [new ASSP/Agent ]
#$s0 report-interval $opt(report_interval)
$r0 attach-node $n5
$r0 set_filename_low  sd_be_00
$r0 set_filename_med  sd_be_11
$r0 set_filename_high sd_be_22
$r0 set_sender true
$r0 set timeout 5
$r0 set low_tx [expr $opt(low)/8]  
$r0 set medium_tx [expr $opt(medium)/8]  
$r0 set high_tx [expr $opt(high)/8]
# enable congestion control
$r0 enable-control 1

set r1 [new ASSP/Sink ]
$r1 attach-node $n12
$r1 set_filename rd_be_11
$r1 set level_ 0
$r1 set_initial_rate  $opt(low)
$r1 set low_threshold [expr $opt(low_threshold)/8] 
$r1 set medium_threshold expr $opt(medium_threshold)/8] 
$r1 set high_threshold [expr $opt(high_threshold)/8]

set r2 [new ASSP/Sink ]
$r2 attach-node $n13
$r2 set_filename rd_be_22
$r2 set level_ 0
$r2 set_initial_rate  $opt(low)
$r2 set low_threshold [expr $opt(low_threshold)/8] 
$r2 set medium_threshold expr $opt(medium_threshold)/8] 
$r2 set high_threshold [expr $opt(high_threshold)/8]

set r3 [new ASSP/Sink ]
$r3 attach-node $n14
$r3 set_filename rd_be_33
$r3 set level_ 0
$r3 set_initial_rate  $opt(high)
$r3 set low_threshold [expr $opt(low_threshold)/8] 
$r3 set medium_threshold expr $opt(medium_threshold)/8] 
$r3 set high_threshold [expr $opt(high_threshold)/8]

set r4 [new ASSP/Sink ]
$r4 attach-node $n15
$r4 set_filename rd_be_44
$r4 set level_ 0
$r4 set_initial_rate  $opt(high)
$r4 set low_threshold [expr $opt(low_threshold)/8] 
$r4 set medium_threshold expr $opt(medium_threshold)/8] 
$r4 set high_threshold [expr $opt(high_threshold)/8]

set r5 [new ASSP/Sink ]
$r5 attach-node $n16
$r5 set_filename rd_be_55
$r5 set level_ 0
$r5 set_initial_rate  $opt(medium)
$r5 set low_threshold [expr $opt(low_threshold)/8] 
$r5 set medium_threshold expr $opt(medium_threshold)/8] 
$r5 set high_threshold [expr $opt(high_threshold)/8]

set r6 [new ASSP/Sink ]
$r6 attach-node $n17
$r6 set_filename rd_be_66
$r6 set level_ 0
$r6 set_initial_rate  $opt(medium)
$r6 set low_threshold [expr $opt(low_threshold)/8] 
$r6 set medium_threshold expr $opt(medium_threshold)/8] 
$r6 set high_threshold [expr $opt(high_threshold)/8]


#################################################
# Create Evalvid-RA traffic sources             #
#################################################
##
## VBR0 as VBR rate adaptiv traffic
##
# The followong file is used as ns-2 adapted (by Chih-Heng, Ke from Taiwan) version of 
# the Evalvid (J. Klaue) generated file above
# Sender trace file from mp4.exe containing frame type and size
puts "Evalvid-RA: Start making GOP and Frame trace files for $q_variants Rate variants"
for {set i 1} {$i <= $q_variants} {incr i} {
    set original_file_name($i) st_highway_cif.yuv_Q[expr $i + 1].txt 
    set original_file_id($i) [open $original_file_name($i) r]
}
set trace_file_name video2.dat
set trace_file_id [open $trace_file_name w]
set trace_file [new vbrTracefile2]
$trace_file filename $trace_file_name
set frame_count 0
set last_time 0

# AL: toggle between multiple input files!
#set original_file_id $original_file_id(1)
#set source_select 1


set frame_size_file frame_size.dat
set frame_size_file_id [open $frame_size_file w]
for {set i 1} {$i <= $q_variants} {incr i} {
    set frame_size($i) 0
}
set gop_size_file gop_size.dat
set gop_size_file_id [open $gop_size_file w]
for {set i 1} {$i <= $q_variants} {incr i} {
    set gop_size($i) 0
}
set gop_numb 0
# Convert ASCII sender file on frame size granularity to ns-2 adapted internal format
while {[eof $original_file_id(1)] == 0} {
    for {set i 1} {$i <= $q_variants} {incr i} {
	gets $original_file_id($i) current_line($i)
    }
    
    scan $current_line(1) "%d%s%d%s%s%s%d%s" no_ frametype_ length_ tmp1_ tmp2_ tmp3_ tmp4_ tmp5_

#    puts "$no_ $frametype_ $length_ $tmp1_ $tmp2_ $tmp3_ $tmp4_ $tmp5_"
        
    # 30 frames/sec. if one want to generate 25 frames/sec, one can use set time [expr 1000*1000/25]
    set tempStr "%.0f"
    set time [expr 1000 * 1000/$frames_per_second] 
#    puts $f5 [format $tempStr $time]
    set time [format $tempStr $time]
    

    if { $frametype_ == "I" } {
  	set type_v 1
	set time 0
    }	

    if { $frametype_ == "P" } {
  	set type_v 2
    }	

    if { $frametype_ == "B" } {
  	set type_v 3
    }	
    
    # Write to GOP size file after each H-frame found:
    if { $frametype_ == "H" } {
	set puts_string "$gop_numb"
	for {set i 1} {$i <= $q_variants} {incr i} {
	    set puts_string "$puts_string $gop_size($i)" 
	}
	puts $gop_size_file_id $puts_string
	set gop_numb [expr $gop_numb + 1]
  	set type_v 0 ;# Must have different type than I-frame so that the LB(r,b) algorithm finds it!
    }	
# Write to frame_size.dat:
    set puts_string "$no_"
    for {set i 1} {$i <= $q_variants} {incr i} {
	set puts_string "$puts_string $gop_size($i)" 
    }
    puts $frame_size_file_id $puts_string

    for {set i 1} {$i <= $q_variants} {incr i} {
	scan $current_line($i) "%d%s%d%s%s%s%d%s" no_ frametype_ length($i) tmp1_ tmp2_ tmp3_ tmp4_ tmp5_
	set gop_size($i) [expr $gop_size($i) + $length($i) ]
    }

# Write to video2.dat:
    set puts_string "$time $length_ $type_v $max_fragmented_size"
    for {set i 2} {$i <= $q_variants} {incr i} {
	set puts_string "$puts_string $length($i)"
    }
    puts  $trace_file_id $puts_string
    incr frame_count
 
}
puts "Evalvid-RA: #of frames written to GOP and Frame trace files: $frame_count"
#close $original_file_id
close $trace_file_id  ;# Note that this new trace file is closed for writing and 
# opened below for reading through being a new Tracefile in eraTraceFile2::setup()



set vbr0 [new Application/Traffic/eraVbrTrace]
	$vbr0 set running_ 0 
	$vbr0 set r_ [expr $opt(low) ]   ;# Set the rate instead of packet interval, 
	#$vbr($i) set r_ [expr $vbr_rate + 0.0]   ;# Set the rate instead of packet interval, 
	
	#$vbr($i) set r_ [expr $vbr_rate + $i*0.5e3]   ;# Set the rate instead of packet interval, 
	;# eraVbrTrace will calculate the interval.
	$vbr0 attach-tracefile $trace_file
	$vbr0 set b_ [expr $opt(low) * 1.0] 
	#$vbr0 set b_ 1.0
	$vbr0 set q_ 12
	#$vbr0 set q_ 2
	$vbr0 set GoP_ 12
	$vbr0 set fps_ $frames_per_second
	$vbr0 set q_level_ 0
	$vbr0 assp-agent $s0
	$s0 attach-vbr $vbr0 

set vbr1 [new Application/Traffic/eraVbrTrace]
	$vbr1 set running_ 0 
	$vbr1 set r_ [expr $opt(medium) ]   ;# Set the rate instead of packet interval, 
	#$vbr($i) set r_ [expr $vbr_rate + 0.0]   ;# Set the rate instead of packet interval, 
	
	#$vbr($i) set r_ [expr $vbr_rate + $i*0.5e3]   ;# Set the rate instead of packet interval, 
	;# eraVbrTrace will calculate the interval.
	$vbr1 attach-tracefile $trace_file
	$vbr1 set b_ [expr $opt(medium) * 1.0] 
	#$vbr1 set b_ 1.0
	$vbr1 set q_ 5
	#$vbr0 set q_ 2
	$vbr1 set GoP_ 12
	$vbr1 set fps_ $frames_per_second
	$vbr1 set q_level_ 1
	$vbr1 assp-agent $s0
	$s0 attach-vbr $vbr1

set vbr2 [new Application/Traffic/eraVbrTrace]
	$vbr2 set running_ 0
	$vbr2 set r_ [expr $opt(high) ]   ;# Set the rate instead of packet interval, 
	#$vbr($i) set r_ [expr $vbr_rate + 0.0]   ;# Set the rate instead of packet interval, 
	
	#$vbr($i) set r_ [expr $vbr_rate + $i*0.5e3]   ;# Set the rate instead of packet interval, 
	;# eraVbrTrace will calculate the interval.
	$vbr2 attach-tracefile $trace_file
	$vbr2 set b_ [expr $opt(high) * 1.0] 
	#$vbr2 set b_ 1.0
	$vbr2 set q_ 2
#	$vbr0 set q_ 2
	$vbr2 set GoP_ 12
	$vbr2 set fps_ $frames_per_second
	$vbr2 set q_level_ 2
	$vbr2 assp-agent $s0
	$s0 attach-vbr $vbr2 


set vbr00 [new Application/Traffic/eraVbrTrace]
	$vbr00 set running_ 0 
	$vbr00 set r_ [expr $opt(low) ]   ;# Set the rate instead of packet interval, 
	#$vbr($i) set r_ [expr $vbr_rate + 0.0]   ;# Set the rate instead of packet interval, 
	
	#$vbr($i) set r_ [expr $vbr_rate + $i*0.5e3]   ;# Set the rate instead of packet interval, 
	;# eraVbrTrace will calculate the interval.
	$vbr00 attach-tracefile $trace_file
	#    $vbr($i) set b_ [expr $vbr_rate * 1.5] 
	$vbr00 set b_ [expr $opt(low) * 1.0] 
	#$vbr00 set b_ 1.0
	$vbr00 set q_ 12
	#$vbr0 set q_ 2
	$vbr00 set GoP_ 12
	$vbr00 set fps_ $frames_per_second
	$vbr00 set q_level_ 0
	$vbr00 assp-agent $r0
	$r0 attach-vbr $vbr00 

set vbr11 [new Application/Traffic/eraVbrTrace]
	$vbr11 set running_ 0 
	$vbr11 set r_ [expr $opt(medium) ]   ;# Set the rate instead of packet interval, 
	#$vbr($i) set r_ [expr $vbr_rate + 0.0]   ;# Set the rate instead of packet interval, 
	
	#$vbr($i) set r_ [expr $vbr_rate + $i*0.5e3]   ;# Set the rate instead of packet interval, 
	;# eraVbrTrace will calculate the interval.
	$vbr11 attach-tracefile $trace_file
	#    $vbr($i) set b_ [expr $vbr_rate * 1.5] 
	$vbr11 set b_ [expr $opt(medium) * 1.0] 
	#$vbr11 set b_ 1.0
	$vbr11 set q_ 5
	#$vbr0 set q_ 2
	$vbr11 set GoP_ 12
	$vbr11 set fps_ $frames_per_second
	$vbr11 set q_level_ 1
	$vbr11 assp-agent $r0
	$r0 attach-vbr $vbr11

set vbr22 [new Application/Traffic/eraVbrTrace]
	$vbr22 set running_ 0
	$vbr22 set r_ [expr $opt(high) ]   ;# Set the rate instead of packet interval, 
	#$vbr($i) set r_ [expr $vbr_rate + 0.0]   ;# Set the rate instead of packet interval, 
	
	#$vbr($i) set r_ [expr $vbr_rate + $i*0.5e3]   ;# Set the rate instead of packet interval, 
	;# eraVbrTrace will calculate the interval.
	$vbr22 attach-tracefile $trace_file
	#    $vbr($i) set b_ [expr $vbr_rate * 1.5] 
	$vbr22 set b_ [expr $opt(high) * 1.0] 
	#$vbr22 set b_ 1.0
	$vbr22 set q_ 2
#	$vbr0 set q_ 2
	$vbr22 set GoP_ 12
	$vbr22 set fps_ $frames_per_second
	$vbr22 set q_level_ 2
	$vbr22 assp-agent $r0
	$r0 attach-vbr $vbr22 
 


$ns at 0.0 "$s0 join-groups $group1 $group2 $group3"
$ns at 0.0 "$s0 start"
#$ns at 0.0 "$s0 start-timer"

$ns at 0.0 "$s1 assign-groups $group1 $group2 $group3"
$ns at 0.0 "$s1 join-group $group1"
$ns at 0.0 "$s1 start-session $group1"

$ns at 0.0 "$s2 assign-groups $group1 $group2 $group3"
$ns at 0.0 "$s2 join-group $group1"
$ns at 0.0 "$s2 start-session $group1"

$ns at 0.0 "$s3 assign-groups $group1 $group2 $group3"
$ns at 0.0 "$s3 join-group $group1"
$ns at 0.0 "$s3 start-session $group1"

$ns at 0.0 "$s4 assign-groups $group1 $group2 $group3"
$ns at 0.0 "$s4 join-group $group1"
$ns at 0.0 "$s4 start-session $group1"

$ns at 0.0 "$s5 assign-groups $group1 $group2 $group3"
$ns at 0.0 "$s5 join-group $group1"
$ns at 0.0 "$s5 start-session $group1"

$ns at 0.0 "$s6 assign-groups $group1 $group2 $group3"
$ns at 0.0 "$s6 join-group $group1"
$ns at 0.0 "$s6 start-session $group1"

$ns at 0.0       "$vbr0 start"
$ns at $simtime  "$vbr0 stop"

$ns at 0.0       "$vbr1 start"
$ns at $simtime  "$vbr1 stop"

$ns at 0.0       "$vbr2 start"
$ns at $simtime  "$vbr2 stop"

############################
#Second ASSP Agent
############################

$ns at 0.0 "$r0 join-groups $group4 $group5 $group6"
$ns at 0.0 "$r0 start"
#$ns at 0.0 "$r0 start-timer"

$ns at 0.0 "$r1 assign-groups $group4 $group5 $group6"
$ns at 0.0 "$r1 join-group $group4"
$ns at 0.0 "$r1 start-session $group4"

$ns at 0.0 "$r2 assign-groups $group4 $group5 $group6"
$ns at 0.0 "$r2 join-group $group4"
$ns at 0.0 "$r2 start-session $group4"

$ns at 0.0 "$r3 assign-groups $group4 $group5 $group6"
$ns at 0.0 "$r3 join-group $group4"
$ns at 0.0 "$r3 start-session $group4"

$ns at 0.0 "$r4 assign-groups $group4 $group5 $group6"
$ns at 0.0 "$r4 join-group $group4"
$ns at 0.0 "$r4 start-session $group4"

$ns at 0.0 "$r5 assign-groups $group4 $group5 $group6"
$ns at 0.0 "$r5 join-group $group4"
$ns at 0.0 "$r5 start-session $group4"

$ns at 0.0 "$r6 assign-groups $group4 $group5 $group6"
$ns at 0.0 "$r6 join-group $group4"
$ns at 0.0 "$r6 start-session $group4"

$ns at 0.0       "$vbr00 start"
$ns at $simtime  "$vbr00 stop"

$ns at 0.0       "$vbr11 start"
$ns at $simtime  "$vbr11 stop"

$ns at 0.0       "$vbr22 start"
$ns at $simtime  "$vbr22 stop"


#===================================
#        Record functions
#===================================

$ns at 0.0 "record-s1"
$ns at 0.0 "record-s3"
$ns at 0.0 "record-s5"

$ns at 0.0 "record-r1"
$ns at 0.0 "record-r3"
$ns at 0.0 "record-r5"

proc record-s1 {} {
        global s1 f1
	#Get an instance of the simulator
	set ns [Simulator instance]
	#Set the time after which the procedure should be called again
        set time 1.5
	#How many bytes have been received by the traffic?
	set bw0 [$s1 set rcv_bytes]
        #Get the current time
        set now [$ns now]
	puts "$now"
#	puts "$now"
	#Calculate the bandwidth (in MBit/s) and write it to the files
	puts $f1 "$now [expr $bw0/$time*8]"
        #Reset the bytes_ values on the traffic sinks
        $s1 set rcv_bytes 0
       	#Re-schedule the procedure
        $ns at [expr $now+$time] "record-s1"
}

proc record-s3 {} {
        global s3 f3
	#Get an instance of the simulator
	set ns [Simulator instance]
	#Set the time after which the procedure should be called again
        set time 1.5
	#How many bytes have been received by the traffic?
	set bw0 [$s3 set rcv_bytes]
        #Get the current time
        set now [$ns now]
	#puts "$now"
#	puts "$now"
	#Calculate the bandwidth (in MBit/s) and write it to the files
	puts $f3 "$now [expr $bw0/$time*8]"
        #Reset the bytes_ values on the traffic sinks
        $s3 set rcv_bytes 0
       	#Re-schedule the procedure
        $ns at [expr $now+$time] "record-s3"
}

proc record-s5 {} {
        global s5 f5
	#Get an instance of the simulator
	set ns [Simulator instance]
	#Set the time after which the procedure should be called again
        set time 1.5
	#How many bytes have been received by the traffic?
	set bw0 [$s5 set rcv_bytes]
        #Get the current time
        set now [$ns now]
	#puts "$now"
#	puts "$now"
	#Calculate the bandwidth (in MBit/s) and write it to the files
	puts $f5 "$now [expr $bw0/$time*8]"
        #Reset the bytes_ values on the traffic sinks
        $s5 set rcv_bytes 0
       	#Re-schedule the procedure
        $ns at [expr $now+$time] "record-s5"
}

proc record-r1 {} {
        global r1 f6
	#Get an instance of the simulator
	set ns [Simulator instance]
	#Set the time after which the procedure should be called again
        set time 1.5
	#How many bytes have been received by the traffic?
	set bw0 [$r1 set rcv_bytes]
        #Get the current time
        set now [$ns now]
	
#	puts "$now"
	#Calculate the bandwidth (in MBit/s) and write it to the files
	puts $f6 "$now [expr $bw0/$time*8]"
        #Reset the bytes_ values on the traffic sinks
        $r1 set rcv_bytes 0
       	#Re-schedule the procedure
        $ns at [expr $now+$time] "record-r1"
}

proc record-r3 {} {
        global r3 f7
	#Get an instance of the simulator
	set ns [Simulator instance]
	#Set the time after which the procedure should be called again
        set time 1.5
	#How many bytes have been received by the traffic?
	set bw0 [$r3 set rcv_bytes]
        #Get the current time
        set now [$ns now]
	
#	puts "$now"
	#Calculate the bandwidth (in MBit/s) and write it to the files
	puts $f7 "$now [expr $bw0/$time*8]"
        #Reset the bytes_ values on the traffic sinks
        $r3 set rcv_bytes 0
       	#Re-schedule the procedure
        $ns at [expr $now+$time] "record-r3"
}

proc record-r5 {} {
        global r5 f8
	#Get an instance of the simulator
	set ns [Simulator instance]
	#Set the time after which the procedure should be called again
        set time 1.5
	#How many bytes have been received by the traffic?
	set bw0 [$r5 set rcv_bytes]
        #Get the current time
        set now [$ns now]
	
#	puts "$now"
	#Calculate the bandwidth (in MBit/s) and write it to the files
	puts $f8 "$now [expr $bw0/$time*8]"
        #Reset the bytes_ values on the traffic sinks
        $r5 set rcv_bytes 0
       	#Re-schedule the procedure
        $ns at [expr $now+$time] "record-r5"
}

#===================================
#        Termination        
#===================================
#Define a 'finish' procedure
proc finish {} {
     global ns tracefile namfile f1 f3 f5 f6 f7 f8
     $ns flush-trace
     close $tracefile
     close $namfile
     close $f1
     close $f3
     close $f5
     close $f6
     close $f7
     close $f8

	 exec xgraph   througput_s1.txt througput_s3.txt througput_s5.txt -geometry 800x400  &
	 exec xgraph   througput_r1.txt througput_r3.txt througput_r5.txt -geometry 800x400  &
     #exec nam out.nam &
     exit 0
}



$ns at [expr $simtime+10] "finish"



$ns run
