#!/bin/bash
#############################################################################
# Script to execute ffmpeg and generate many versions of VBR open loop video.
# Input arguments: 
# $1: Quantizer minimum. 
# $2: Quantizer maximum
# $3: input file name (the video file that ffmpeg knows how to decode, i.e. trascode)
#
# Output is a *.m4v file (mpeg4 video bit stream) file
#
# Then this file is stripped for eventially repeated headers using 
#            mp4.exe -strip
# Then the send trace is generated with 
#            mp4.exe -send
# from Evalvid.
# Script born Tuesday 28.06.05.
############################################################################
# How to run manyQ.sh from directory of video, manyQ.sh and the tcl script:
#
# command_prompt> ./manyQ.sh <min Q-scale> <max Q-scale> <videofile.mov>
#
# The input video file must be of format that ffmpeg can process. Can be raw YUV also.
# NOTE: manyQ.sh may have to be modified, ffmpeg might not support the 
# -4mv and -s_gop switch. If this is the case, delete these switches from 
# the line starting with 
#     ffmpeg -s cif -r 30 -vframes 1836...
# Note also that the same line must be customized to frame size, frame rate, etc.
# The GOP size is default 12. If modified, also the et_ra.exe source files
# should be modified: change the #define GOP_SIZE_ 12 to whatever number is desired.

#MP4exe=~/Evalvid/mp4-1.2/mp4.exe
MP4=/root/Desktop/wired-part/multi-evalvid-ra/mp4

###############################################################
# Some tutorial script lines are added for education purposes
###############################################################
#clear
#echo "Hello $USER"
#echo -e "Today is \c ";date
#echo -e "Number of user login : \c" ; who | wc -l
#echo "Calendar"
#cal

#echo "Your first name please:"
#read fname
#echo "Hello $fname, let's be friends!"

#expr 1 + 3

#echo "Total number of command line argument are $#"
#echo "$0 is script name"
#echo "$1 is first argument"
#echo "$2 is second argument"
#echo "All of them are :- $* or $@"

#if test $1 -gt 0
#then
#    echo "$1 number is positive"
#elif [ $1 -lt 0 ]
#then
#  echo "$1 is negative"
#elif [ $1 -eq 0 ]
#then
#  echo "$1 is zero"
#else
#  echo "Oups! $1 is not a number!"
#fi

######################################
# Work begins...
#
# ffmpeg parameters:
# (Note that parameters can reflect input and output files. The file the switch belongs
# to is the first file mentioned right of the switch.)
# -r is fps, and is set twice: forces 25fps of input, and 25fps of output
# -s is set to CIF
# -vcodec is set to mpeg4 (format of output)
# -qscale is fixed quantizer (==> VBR open loop)
# -g is GOP size in numb. of frames. Note that ffmpeg might modify this to 
#    shorter GOPs when needed.
# -sgop Use this one to force STATIC GOP size. Might not be supported in newer ffmepg versions. If so, remove this below!
# -y forces output file overwrite in case it exists.
# -vframes <number> is for limiting processing to a certain number of frames
# -4mv MPEG-4 motion vector. Might be skipped
# -sc_threshold A parameter to adjust the ffmpeg's eager to modify GOP size base on scene changes. Might be obsolete.
#
# mp4.exe parameters:
# -strip searches input file for repeated headers and removes them for output
# -send <stream dest. IP address> <port numb> <fps> <source file> > <stream data>
# Note: I have speeded up this function by indicating very high frame rate!! The time
# schedule will nevertheless be overruled by the ns-2 program.
######################################
for ((  i=$1 ;  i<=$2;  i++  ))
do
#    ffmpeg -r 25 -i $3 -vcodec mpeg4 -4mv -g 12 -sgop -sc_threshold 10000 -qscale $i -s 352x288 -r 25 -y $3_Q$i.m4v
# I USE THE NEXT LINE FOR .YUV FILES. this is also my default GK
#    ffmpeg -s cif -r 25 -vframes 1836 -i $3 -vcodec mpeg4  -g 12  -sc_threshold 20000 -qscale $i -s 352x288 -r 25 -y $3_Q$i.m4v
# include B-Frames
ffmpeg -s cif -r 25 -vframes 300 -i $3 -bf 1 -vcodec mpeg4  -g 12  -sc_threshold 20000 -qscale $i -s 352x288 -r 25 -y $3_Q$i.m4v
    
# If using ffmpeg rate controller:
# ffmpeg -s cif -r 30 -vframes 1836 -i $3 -vcodec mpeg4 -4mv -g 12 -s cif -r 30 -b 600 -minrate 500 -maxrate 700 -y $3_Q$i.m4v
# Run -strip if removing repeated headers. But I need them!!!
    #$MP4exe -strip $3_Q$i.m4v $3_Qs$i.m4v 
    #rm $3_Q$i.m4v
#    $MP4exe -send 192.168.1.100 1234 1000 25 $3_Q$i.m4v > st_$3_Q$i.txt
    $MP4 -send 224.168.1.100 1234 1000 2500 $3_Q$i.m4v > st_$3_Q$i.txt
done

##############################################################################
# That completes the pre-processing.
#
# Now, run the network simulation:
#ns tfrc_vbr_simple.tcl

# The Post-processing follows: see runPoD.sh and runPoB.sh for details!!
###############################################################################

exit 0
